@@ -36,6 +36,8 @@ using ICSharpCode.NRefactory.Parser;
 using ICSharpCode.NRefactory.Ast;
 using ICSharpCode.NRefactory;
 using MonoDevelop.Refactoring;
+using MonoDevelop.CSharp.Completion;
+using MonoDevelop.Projects.Dom.Output;
 
 namespace MonoDevelop.CSharp.Resolver
 {
@@ -59,6 +61,21 @@ namespace MonoDevelop.CSharp.Resolver
 		public IReturnType GetTypeSafe (Expression expression)
 		{
 			ResolveResult result = Resolve (expression);
+			if (expression is LambdaExpression) {
+				var lambda = (LambdaExpression)expression; 
+				var bodyType = GetTypeSafe (lambda.ExpressionBody);
+				DomReturnType constructedLambdaType = new DomReturnType (bodyType.FullName == DomReturnType.Void.FullName ? "System.Action" : "System.Func");
+				foreach (var param in lambda.Parameters) {
+					var typeParam = GetTypeSafe (param);
+					// add void place holder for types that can't be resolved.
+					if (typeParam == null || string.IsNullOrEmpty (typeParam.FullName))
+						typeParam = DomReturnType.Void;
+					constructedLambdaType.AddTypeParameter (typeParam);
+				}
+				if (bodyType.FullName != DomReturnType.Void.FullName)
+					constructedLambdaType.AddTypeParameter (bodyType ?? result.ResolvedType);
+				return constructedLambdaType;
+			}
 			return result.ResolvedType ?? DomReturnType.Void;
 		}
 		
@@ -85,9 +102,7 @@ namespace MonoDevelop.CSharp.Resolver
 			result.ResolvedType = type;
 			result.UnresolvedType = type;
 			if (unit != null && resolver.Dom != null && type != null && type.Type == null) {
-				SearchTypeRequest req = new SearchTypeRequest (unit, type, resolver.CallingType);
-				req.CallingType = resolver.CallingType;
-				IType searchedType = resolver.Dom.SearchType (req);
+				IType searchedType = resolver.SearchType (type);
 				if (searchedType != null) {
 					DomReturnType resType = new DomReturnType (searchedType);
 					resType.ArrayDimensions = type.ArrayDimensions;
@@ -105,14 +120,17 @@ namespace MonoDevelop.CSharp.Resolver
 		{
 			if (identifierExpression.TypeArguments != null && identifierExpression.TypeArguments.Count > 0) {
 				if (resolver.CallingType != null) {
-					IMethod possibleMethod = resolver.CallingType.Methods.Where (m => m.Name == identifierExpression.Identifier && m.TypeParameters.Count == identifierExpression.TypeArguments.Count).FirstOrDefault ();
-					if (possibleMethod != null) {
-						MethodResolveResult methodResolveResult = new MethodResolveResult (possibleMethod);
-						methodResolveResult.CallingType   = resolver.CallingType;
-						methodResolveResult.CallingMember = resolver.CallingMember;
-						identifierExpression.TypeArguments.ForEach (arg => methodResolveResult.AddGenericArgument (arg.ConvertToReturnType ()));
-						methodResolveResult.ResolveExtensionMethods ();
-						return methodResolveResult;
+					foreach (var type in resolver.Dom.GetInheritanceTree (resolver.CallingType)) {
+						IMethod possibleMethod = type.Methods.Where (m => m.Name == identifierExpression.Identifier && m.TypeParameters.Count == identifierExpression.TypeArguments.Count && m.IsAccessibleFrom (resolver.Dom, resolver.CallingType, resolver.CallingMember, true)).FirstOrDefault ();
+						if (possibleMethod != null) {
+							MethodResolveResult methodResolveResult = new MethodResolveResult (possibleMethod);
+							methodResolveResult.CallingType   = resolver.CallingType;
+							methodResolveResult.CallingMember = resolver.CallingMember;
+							
+							identifierExpression.TypeArguments.ForEach (arg => methodResolveResult.AddGenericArgument (resolver.ResolveType (arg.ConvertToReturnType ())));
+							methodResolveResult.ResolveExtensionMethods ();
+							return methodResolveResult;
+						}
 					}
 				}
 				TypeReference reference = new TypeReference (identifierExpression.Identifier);
@@ -121,9 +139,9 @@ namespace MonoDevelop.CSharp.Resolver
 				result.StaticResolve = true;
 				return result;
 			}
-			//Console.WriteLine ("visit id: " + identifierExpression.Identifier);
+//			Console.WriteLine ("visit id: " + identifierExpression.Identifier);
 			var res = resolver.ResolveIdentifier (this, identifierExpression.Identifier.TrimEnd ('.'));
-			//Console.WriteLine ("result: " + res);
+//			Console.WriteLine ("result: " + res);
 			return res;
 		}
 		
@@ -153,8 +171,27 @@ namespace MonoDevelop.CSharp.Resolver
 		{
 			if (collectionInitializerExpression.CreateExpressions.Count == 0)
 				return null;
-			DomReturnType type = (DomReturnType)ResolveType (collectionInitializerExpression.CreateExpressions[0]);
-			type.ArrayDimensions++;
+			DomReturnType type = null;
+			IType typeObject = null;
+			
+			for (int i = 0; i < collectionInitializerExpression.CreateExpressions.Count; i++) {
+				DomReturnType curType = (DomReturnType)ResolveType (collectionInitializerExpression.CreateExpressions[i]);
+				// if we found object or we have only one create expression we can stop
+				if (curType.DecoratedFullName == DomReturnType.Object.DecoratedFullName || collectionInitializerExpression.CreateExpressions.Count == 1) {
+					type = curType;
+					break;
+				}
+				IType curTypeObject = resolver.Dom.GetType (curType);
+				if (curTypeObject == null)
+					continue;
+				if (type == null || resolver.Dom.GetInheritanceTree (typeObject).Any (t => t.DecoratedFullName == curTypeObject.DecoratedFullName)) {
+					type = curType;
+					typeObject = curTypeObject;
+				}
+			}
+			
+			if (type != null)
+				type.ArrayDimensions++;
 			return CreateResult (type);
 		}
 		
@@ -204,14 +241,22 @@ namespace MonoDevelop.CSharp.Resolver
 			return Resolve (unaryOperatorExpression.Expression);
 		}
 		
+		public override object VisitDefaultValueExpression (DefaultValueExpression defaultValueExpression, object data)
+		{
+			return CreateResult (defaultValueExpression.TypeReference);
+		}
+		
 		public override object VisitIndexerExpression(IndexerExpression indexerExpression, object data)
 		{
 			if (indexerExpression.Indexes == null || indexerExpression.Indexes.Count == 0)
 				return null;
 			ResolveResult result = Resolve (indexerExpression.TargetObject);
 			
-			if (result.ResolvedType != null && result.ResolvedType.ArrayDimensions > 0)
-				return CreateResult (result.ResolvedType.FullName);
+			if (result.ResolvedType != null && result.ResolvedType.ArrayDimensions > 0) {
+				((DomReturnType)result.ResolvedType).ArrayDimensions--;
+				return CreateResult (result.ResolvedType);
+			}
+			
 			IType resolvedType = resolver.Dom.GetType (result.ResolvedType);
 			if (resolvedType != null) {
 				foreach (IType curType in resolver.Dom.GetInheritanceTree (resolvedType)) {
@@ -224,7 +269,6 @@ namespace MonoDevelop.CSharp.Resolver
 				}
 			}
 			if (result.ResolvedType != null && result.ResolvedType.GenericArguments.Count > 0) {
-				//System.Console.WriteLine("genArg:" + result.ResolvedType.GenericArguments[0]);
 				return CreateResult (result.ResolvedType.GenericArguments[0]);
 			}
 			return result;
@@ -242,15 +286,23 @@ namespace MonoDevelop.CSharp.Resolver
 			return "?";
 		}
 		
+		Dictionary<CollectionInitializerExpression, DomType> anonymousTypes = new Dictionary<CollectionInitializerExpression, DomType> ();
 		IType CreateAnonymousClass (CollectionInitializerExpression initializer)
 		{
-			DomType result = new AnonymousType ();
-			result.SourceProjectDom = resolver.Dom;
-			foreach (Expression expr in initializer.CreateExpressions) {
-				DomProperty newProperty = new DomProperty (GetAnonymousTypeFieldName (expr), MonoDevelop.Projects.Dom.Modifiers.Public, DomLocation.Empty, DomRegion.Empty, ResolveType(expr));
-				newProperty.Modifiers = MonoDevelop.Projects.Dom.Modifiers.Public;
-				newProperty.DeclaringType = result;
-				result.Add (newProperty);
+			DomType result;
+			if (!anonymousTypes.TryGetValue (initializer, out result)) {
+				result = new AnonymousType ();
+				result.SourceProjectDom = resolver.Dom;
+				foreach (Expression expr in initializer.CreateExpressions) {
+					var oldPos = resolver.ResolvePosition;
+					if (!expr.StartLocation.IsEmpty)
+						resolver.resolvePosition = new DomLocation (expr.StartLocation.Line + resolver.CallingMember.Location.Line, expr.StartLocation.Column);
+					DomProperty newProperty = new DomProperty (GetAnonymousTypeFieldName (expr), MonoDevelop.Projects.Dom.Modifiers.Public, DomLocation.Empty, DomRegion.Empty, ResolveType (expr));
+					newProperty.DeclaringType = result;
+					result.Add (newProperty);
+					resolver.resolvePosition = oldPos;
+				}
+				anonymousTypes[initializer] = result;
 			}
 			return result;
 		}
@@ -411,6 +463,7 @@ namespace MonoDevelop.CSharp.Resolver
 				return CreateResult (DomReturnType.Void);
 				
 			ThisResolveResult result = new ThisResolveResult ();
+			
 			result.CallingType   = resolver.CallingType;
 			result.CallingMember = resolver.CallingMember;
 			result.UnresolvedType = result.ResolvedType  = DomReturnType.GetSharedReturnType (new DomReturnType (resolver.CallingType));
@@ -428,7 +481,7 @@ namespace MonoDevelop.CSharp.Resolver
 			if (resolver.CallingType != null) {
 				IType type = null;
 				if (resolver.CallingType.BaseType != null) 
-					type = this.resolver.Dom.SearchType (new SearchTypeRequest (resolver.Unit, resolver.CallingType.BaseType, resolver.CallingType));
+					type = this.resolver.SearchType (resolver.CallingType.BaseType);
 				result.UnresolvedType = result.ResolvedType  = type != null ? new DomReturnType (type) : DomReturnType.Object;
 			}
 			return result;
@@ -476,7 +529,7 @@ namespace MonoDevelop.CSharp.Resolver
 				//				return memberReferenceExpression.TargetObject.AcceptVisitor(this, data);
 			}
 			result = memberReferenceExpression.TargetObject.AcceptVisitor (this, data) as ResolveResult;
-
+			
 			NamespaceResolveResult namespaceResult = result as NamespaceResolveResult;
 			if (namespaceResult != null) {
 				if (String.IsNullOrEmpty (memberReferenceExpression.MemberName))
@@ -498,84 +551,117 @@ namespace MonoDevelop.CSharp.Resolver
 				}
 				return null;
 			}
-
 			if (result != null && result.ResolvedType != null) {
-				IType type = resolver.Dom.GetType (result.ResolvedType);
-				if (type != null) {
-					List<IMember> member = new List<IMember> ();
-					List<IType> accessibleExtTypes = DomType.GetAccessibleExtensionTypes (resolver.Dom, resolver.Unit);
-					// Inheritance of extension methods is handled in DomType
-					foreach (IMethod method in type.GetExtensionMethods (accessibleExtTypes)) {
-						if (method.Name == memberReferenceExpression.MemberName) {
-							member.Add (method);
-						}
-					}
-					bool includeProtected = true;
-					foreach (IType curType in resolver.Dom.GetInheritanceTree (type)) {
-						if (curType.ClassType == MonoDevelop.Projects.Dom.ClassType.Interface && type.ClassType != MonoDevelop.Projects.Dom.ClassType.Interface)
-							continue;
-						if (curType.IsAccessibleFrom (resolver.Dom, resolver.CallingType, resolver.CallingMember, includeProtected)) {
-							foreach (IMember foundMember in curType.SearchMember (memberReferenceExpression.MemberName, true)) {
-								if (foundMember.IsExplicitDeclaration)
-									continue;
-								member.Add (foundMember);
-							}
-						}
-					}
-					if (member.Count > 0) {
-						if (member[0] is IMethod) {
-							bool isStatic = result.StaticResolve;
-							List<IMember> nonMethodMembers = new List<IMember> ();
-							for (int i = 0; i < member.Count; i++) {
-								IMethod method = member[i] as IMethod;
-								if (method == null)
-									nonMethodMembers.Add (member[i]);
-								if (method != null && !method.IsFinalizer && (method.IsExtension || method.WasExtended) && method.IsAccessibleFrom (resolver.Dom, type, resolver.CallingMember, true))
-									continue;
-								if ((member[i].IsStatic ^ isStatic) || !member[i].IsAccessibleFrom (resolver.Dom, type, resolver.CallingMember, includeProtected) || (method != null && method.IsFinalizer)) {
-									member.RemoveAt (i);
-									i--;
-								}
-							}
-							if (member.Count == 0)
-								return null;
-							result = new MethodResolveResult (member);
-							((MethodResolveResult)result).Type = type;
-							result.CallingType   = resolver.CallingType;
-							result.CallingMember = resolver.CallingMember;
-							//result.StaticResolve = isStatic;
-							//result.UnresolvedType = result.ResolvedType  = member[0].ReturnType;
-							foreach (TypeReference typeReference in memberReferenceExpression.TypeArguments) {
-								((MethodResolveResult)result).AddGenericArgument (typeReference.ConvertToReturnType ());
-							}
-							((MethodResolveResult)result).ResolveExtensionMethods ();
-							if (nonMethodMembers.Count > 0) {
-								IType searchType = resolver.Dom.GetType (nonMethodMembers[0].ReturnType);
-								MemberResolveResult baseResult = (MemberResolveResult) CreateResult (nonMethodMembers[0].DeclaringType.CompilationUnit, searchType != null ? new DomReturnType (searchType) : DomReturnType.Void);
-								baseResult.ResolvedMember = nonMethodMembers[0];
-								return new CombinedMethodResolveResult (baseResult, (MethodResolveResult)result);
-							}
-							//System.Console.WriteLine(result + "/" + result.ResolvedType);
-							return result;
-						}
-						
-						if (member[0] is IType) {
-							result = CreateResult (member[0].FullName);
-							result.StaticResolve = true;
-						} else {
-							IType searchType = resolver.Dom.GetType (member[0].ReturnType);
-							result = CreateResult (member[0].DeclaringType.CompilationUnit, searchType != null ? new DomReturnType (searchType) : DomReturnType.Void);
-							((MemberResolveResult)result).ResolvedMember = member[0];
-						}
-						return result;
-					}
-				} else {
-					MonoDevelop.Core.LoggingService.LogWarning ("Couldn't resolve type " + result.ResolvedType);
+				foreach (ResolveResult innerResult in result.ResolveResults) {
+					ResolveResult resolvedResult = ResolveMemberReference (innerResult, memberReferenceExpression);
+					if (resolvedResult != null)
+						return resolvedResult;
 				}
+			} else {
+				if (result != null)
+					MonoDevelop.Core.LoggingService.LogWarning ("Couldn't resolve type " + result);
 			}
 			
 			return null;
 		}
+		static MonoDevelop.CSharp.Ast.CSharpAmbience ambience = new MonoDevelop.CSharp.Ast.CSharpAmbience ();
+		ResolveResult ResolveMemberReference (ResolveResult result, MemberReferenceExpression memberReferenceExpression)
+		{
+			IType type = resolver.Dom.GetType (result.ResolvedType);
+			if (type == null) 
+				return null;
+			//Console.WriteLine ("Resolve member: " + memberReferenceExpression.MemberName + " on " + type);
+			
+			List<IMember> member = new List<IMember> ();
+			List<IType> accessibleExtTypes = DomType.GetAccessibleExtensionTypes (resolver.Dom, resolver.Unit);
+			// Inheritance of extension methods is handled in DomType
+			foreach (IMethod method in type.GetExtensionMethods (accessibleExtTypes)) {
+				if (method.Name == memberReferenceExpression.MemberName) {
+					member.Add (method);
+				}
+			}
+			bool includeProtected = true;
+			foreach (IType curType in resolver.Dom.GetInheritanceTree (type)) {
+				if (curType.ClassType == MonoDevelop.Projects.Dom.ClassType.Interface && type.ClassType != MonoDevelop.Projects.Dom.ClassType.Interface)
+					continue;
+				if (curType.IsAccessibleFrom (resolver.Dom, resolver.CallingType, resolver.CallingMember, includeProtected)) {
+					foreach (IMember foundMember in curType.SearchMember (memberReferenceExpression.MemberName, true)) {
+						if (foundMember.IsExplicitDeclaration)
+							continue;
+						if (result is BaseResolveResult && foundMember.IsAbstract)
+							continue;
+						member.Add (foundMember);
+					}
+				} 
+			}
+			if (member.Count > 0) {
+				if (member [0] is IMethod) {
+					bool isStatic = result.StaticResolve;
+					List<IMember> nonMethodMembers = new List<IMember> ();
+					List<string> errors = new List<string> ();
+					int typeParameterCount = 0;
+					if (memberReferenceExpression.TypeArguments != null)
+						typeParameterCount = memberReferenceExpression.TypeArguments.Count;
+					
+					for (int i = 0; i < member.Count; i++) {
+						IMethod method = member [i] as IMethod;
+						if (method == null)
+							nonMethodMembers.Add (member [i]);
+						
+						if (!member [i].IsAccessibleFrom (resolver.Dom, type, resolver.CallingMember, includeProtected))
+							errors.Add (
+								MonoDevelop.Core.GettextCatalog.GetString ("'{0}' is inaccessible due to its protection level.", 
+								ambience.GetString (method, OutputFlags.IncludeParameters | OutputFlags.IncludeGenerics)));
+						
+						if (method != null && !method.IsFinalizer && (method.IsExtension || method.WasExtended)/* && method.IsAccessibleFrom (resolver.Dom, type, resolver.CallingMember, true)*/) {
+							continue;
+						}
+						if ((member [i].IsStatic ^ isStatic) || 
+						/*						    !member[i].IsAccessibleFrom (resolver.Dom, type, resolver.CallingMember, includeProtected) || */
+						    (method != null && (method.IsFinalizer || typeParameterCount > 0 && method.TypeParameters.Count != typeParameterCount))) {
+							member.RemoveAt (i);
+							i--;
+						}
+					}
+					if (member.Count == 0)
+						return null;
+					result = new MethodResolveResult (member);
+					((MethodResolveResult)result).Type = type;
+					((MethodResolveResult)result).StaticUsage = isStatic;
+					result.CallingType = resolver.CallingType;
+					result.CallingMember = resolver.CallingMember;
+					result.ResolveErrors.AddRange (errors);
+					result.StaticResolve = isStatic;
+					//result.UnresolvedType = result.ResolvedType  = member[0].ReturnType;
+					foreach (TypeReference typeReference in memberReferenceExpression.TypeArguments) {
+						((MethodResolveResult)result).AddGenericArgument (resolver.ResolveType (typeReference.ConvertToReturnType ()));
+					}
+					((MethodResolveResult)result).ResolveExtensionMethods ();
+					if (nonMethodMembers.Count > 0) {
+						MemberResolveResult baseResult = (MemberResolveResult)CreateResult (nonMethodMembers [0].DeclaringType.CompilationUnit, nonMethodMembers [0].ReturnType);
+						baseResult.ResolvedMember = nonMethodMembers [0];
+						return new CombinedMethodResolveResult (baseResult, (MethodResolveResult)result);
+					}
+					//System.Console.WriteLine(result + "/" + result.ResolvedType);
+					return result;
+				}
+				if (member [0] is IType) {
+					result = CreateResult (member [0].FullName);
+					result.StaticResolve = true;
+				} else {
+					result = CreateResult (member [0].DeclaringType.CompilationUnit, member [0].ReturnType);
+					((MemberResolveResult)result).ResolvedMember = member [0];
+				}
+				if (!member [0].IsAccessibleFrom (resolver.Dom, type, resolver.CallingMember, includeProtected))
+					result.ResolveErrors.Add (string.Format (MonoDevelop.Core.GettextCatalog.GetString ("'{0}' is inaccessible due to it's protection level."), ambience.GetString (member [0], OutputFlags.IncludeParameters | OutputFlags.IncludeGenerics)));
+			
+				return result;
+			}
+			return new UnresolvedMemberResolveResult (result, memberReferenceExpression.MemberName) {
+				CallingType   = resolver.CallingType,
+				CallingMember = resolver.CallingMember
+			};
+		}
 		
 		Dictionary<InvocationExpression, ResolveResult> invocationDictionary = new Dictionary<InvocationExpression, ResolveResult> ();
 		public void ResetVisitor ()
@@ -600,10 +686,12 @@ namespace MonoDevelop.CSharp.Resolver
 				if (idExpr.Identifier == "__reftype") 
 					return CreateResult ("System.Type");
 			}
+			
 			ResolveResult targetResult = Resolve (invocationExpression.TargetObject);
 			if (targetResult is CombinedMethodResolveResult)
 				targetResult = ((CombinedMethodResolveResult)targetResult).MethodResolveResult;
-			targetResult.StaticResolve = false; // invocation result is never static
+			bool targetIsStatic = targetResult.StaticResolve;
+			targetResult.StaticResolve = false; // invocation results are never static because they return objects
 			if (this.resolver.CallingType != null) {
 				if (targetResult is ThisResolveResult) {
 					targetResult = new MethodResolveResult (this.resolver.CallingType.Methods.Where (method => method.IsConstructor));
@@ -612,11 +700,35 @@ namespace MonoDevelop.CSharp.Resolver
 					targetResult.CallingMember = resolver.CallingMember;
 				} else if (targetResult is BaseResolveResult) {
 					System.Collections.IEnumerable baseConstructors = null;
-					foreach (IType baseType in resolver.Dom.GetInheritanceTree (this.resolver.CallingType)) {
-						if (baseType.DecoratedFullName == this.resolver.CallingType.DecoratedFullName || baseType.ClassType == MonoDevelop.Projects.Dom.ClassType.Interface)
-							continue;
-						baseConstructors = baseType.Methods.Where (method => method.IsConstructor);
-						break;
+					IType firstBaseType = null;
+					foreach (IReturnType bT in this.resolver.CallingType.BaseTypes) {
+						IType resolvedBaseType = resolver.SearchType (bT);
+						if (firstBaseType == null && resolvedBaseType.ClassType != MonoDevelop.Projects.Dom.ClassType.Interface)
+							firstBaseType = resolvedBaseType;
+						foreach (IType baseType in resolver.Dom.GetInheritanceTree (resolvedBaseType)) {
+							if (baseType.ClassType == MonoDevelop.Projects.Dom.ClassType.Interface)
+								break;
+							baseConstructors = baseType.Methods.Where (method => method.IsConstructor);
+							goto bailOut;
+						}
+					}
+				bailOut:
+					if (baseConstructors == null) {
+						if (firstBaseType != null) {
+							// if there is a real base type without a .ctor a default .ctor for this type is generated.
+							DomMethod constructedConstructor;
+							constructedConstructor = new DomMethod ();
+							constructedConstructor.Name = ".ctor";
+							constructedConstructor.MethodModifier = MethodModifier.IsConstructor;
+							constructedConstructor.DeclaringType = firstBaseType;
+							constructedConstructor.Modifiers = MonoDevelop.Projects.Dom.Modifiers.Public;
+							baseConstructors = new IMethod[] {
+								constructedConstructor
+							};
+						} else {
+							baseConstructors = resolver.SearchType (DomReturnType.Object).SearchMember (".ctor", true);
+						}
+						
 					}
 					targetResult = new MethodResolveResult (baseConstructors);
 					((MethodResolveResult)targetResult).Type = this.resolver.CallingType;
@@ -627,6 +739,7 @@ namespace MonoDevelop.CSharp.Resolver
 			
 			MethodResolveResult methodResult = targetResult as MethodResolveResult;
 			if (methodResult != null) {
+				methodResult.GetsInvoked = true;
 //				Console.WriteLine ("--------------------");
 //				Console.WriteLine ("i:" + methodResult.ResolvedType);
 /*				foreach (var arg in methodResult.GenericArguments) {
@@ -637,6 +750,7 @@ namespace MonoDevelop.CSharp.Resolver
 					methodResult.AddArgument (type);
 				}
 				//Console.WriteLine ("--------------------");
+				methodResult.StaticUsage = targetIsStatic;
 				methodResult.ResolveExtensionMethods ();
 //				Console.WriteLine ("i2:" + methodResult.ResolvedType);
 			/*	MemberReferenceExpression mre = invocationExpression.TargetObject as MemberReferenceExpression;
@@ -697,7 +811,7 @@ namespace MonoDevelop.CSharp.Resolver
 				selectLambdaExpr.ExpressionBody = selectClause.Projection;
 				TypeReference typeRef = new TypeReference ("System.Func");
 				typeRef.GenericTypes.Add (TypeReference.Null);
-				ResolveResult result = resolver.ResolveExpression (selectLambdaExpr, resolver.ResolvePosition);
+				ResolveResult result = resolver.ResolveExpression (selectLambdaExpr, resolver.ResolvePosition, false);
 				
 				typeRef.GenericTypes.Add (result.ResolvedType.ConvertToTypeReference ());
 				
