@@ -43,9 +43,9 @@ namespace MonoDevelop.Debugger
 			this.editor = editor;
 		}
 		
-		public void DrawIcon (Mono.TextEditor.TextEditor editor, Gdk.Drawable win, LineSegment line, int lineNumber, int x, int y, int width, int height)
+		public void DrawIcon (Mono.TextEditor.TextEditor editor, Cairo.Context cr, LineSegment line, int lineNumber, double x, double y, double width, double height)
 		{
-			int size;
+			double size;
 			if (width > height) {
 				x += (width - height) / 2;
 				size = height;
@@ -54,16 +54,14 @@ namespace MonoDevelop.Debugger
 				size = width;
 			}
 			
-			using (Cairo.Context cr = Gdk.CairoHelper.Create (win)) {
-				DrawIcon (cr, x, y, size);
-			}
+			DrawIcon (cr, x, y, size);
 		}
 		
-		protected virtual void DrawIcon (Cairo.Context cr, int x, int y, int size)
+		protected virtual void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
 		}
 		
-		protected void DrawCircle (Cairo.Context cr, double x, double y, int size)
+		protected void DrawCircle (Cairo.Context cr, double x, double y, double size)
 		{
 			x += 0.5; y += 0.5;
 			cr.NewPath ();
@@ -104,22 +102,23 @@ namespace MonoDevelop.Debugger
 			cr.ClosePath ();
 		}
 		
-		protected void FillGradient (Cairo.Context cr, Cairo.Color color1, Cairo.Color color2, int x, int y, int size)
+		protected void FillGradient (Cairo.Context cr, Cairo.Color color1, Cairo.Color color2, double x, double y, double size)
 		{
-			Cairo.Gradient pat = new Cairo.LinearGradient (x + size / 4, y, x + size / 2, y + size - 4);
-			pat.AddColorStop (0, color1);
-			pat.AddColorStop (1, color2);
-			cr.Pattern = pat;
-			cr.FillPreserve ();
+			using (var pat = new Cairo.LinearGradient (x + size / 4, y, x + size / 2, y + size - 4)) {
+				pat.AddColorStop (0, color1);
+				pat.AddColorStop (1, color2);
+				cr.Pattern = pat;
+				cr.FillPreserve ();
+			}
 		}
 		
-		protected void DrawBorder (Cairo.Context cr, Cairo.Color color, int x, int y, int size)
+		protected void DrawBorder (Cairo.Context cr, Cairo.Color color, double x, double y, double size)
 		{
-			Cairo.Gradient pat = new Cairo.LinearGradient (x, y + size, x + size, y);
-			pat.AddColorStop (0, color);
-			cr.Pattern = pat;
-			cr.LineWidth = 1;
-			cr.Stroke ();
+			using (var pat = new Cairo.LinearGradient (x, y + size, x + size, y)) {
+				pat.AddColorStop (0, color);
+				cr.Pattern = pat;
+				cr.Stroke ();
+			}
 		}
 
 		public void MousePress (MarginMouseEventArgs args)
@@ -129,15 +128,19 @@ namespace MonoDevelop.Debugger
 		public void MouseRelease (MarginMouseEventArgs args)
 		{
 		}
+		
+		public void MouseHover (MarginMouseEventArgs args)
+		{
+		}
 	}
 	
 	public class BreakpointTextMarker : DebugTextMarker
 	{
-		public override Color BackgroundColor {
+		public override Cairo.Color BackgroundColor {
 			get { return editor.ColorStyle.BreakpointBg; }
 			set {  }
 		}
-		public override Color Color {
+		public override Cairo.Color Color {
 			get { return editor.ColorStyle.BreakpointFg; }
 			set {  }
 		}
@@ -150,10 +153,10 @@ namespace MonoDevelop.Debugger
 			IsTracepoint = isTracePoint;
 		}
 		
-		protected override void DrawIcon (Cairo.Context cr, int x, int y, int size)
+		protected override void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
-			Cairo.Color color1 = Style.ToCairoColor (editor.ColorStyle.BreakpointMarkerColor1);
-			Cairo.Color color2 = Style.ToCairoColor (editor.ColorStyle.BreakpointMarkerColor2);
+			Cairo.Color color1 = editor.ColorStyle.BreakpointMarkerColor1;
+			Cairo.Color color2 = editor.ColorStyle.BreakpointMarkerColor2;
 			if (IsTracepoint)
 				DrawDiamond (cr, x, y, size);
 			else
@@ -165,7 +168,7 @@ namespace MonoDevelop.Debugger
 	
 	public class DisabledBreakpointTextMarker: DebugTextMarker
 	{
-		public override Color BackgroundColor {
+		public override Cairo.Color BackgroundColor {
 			get { return editor.ColorStyle.DisabledBreakpointBg; }
 			set {  }
 		}
@@ -178,9 +181,9 @@ namespace MonoDevelop.Debugger
 		
 		public bool IsTracepoint { get; set; }
 		
-		protected override void DrawIcon (Cairo.Context cr, int x, int y, int size)
+		protected override void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
-			Cairo.Color border = Style.ToCairoColor (editor.ColorStyle.InvalidBreakpointMarkerBorder);
+			Cairo.Color border = editor.ColorStyle.InvalidBreakpointMarkerBorder;
 			if (IsTracepoint)
 				DrawDiamond (cr, x, y, size);
 			else
@@ -192,12 +195,12 @@ namespace MonoDevelop.Debugger
 	
 	public class CurrentDebugLineTextMarker: DebugTextMarker
 	{
-		public override Color BackgroundColor {
+		public override Cairo.Color BackgroundColor {
 			get { return editor.ColorStyle.CurrentDebugLineBg; }
 			set {  }
 		}
 		
-		public override Color Color {
+		public override Cairo.Color Color {
 			get { return editor.ColorStyle.CurrentDebugLineFg;  }
 			set {  }
 		}
@@ -207,11 +210,11 @@ namespace MonoDevelop.Debugger
 			IncludedStyles |= StyleFlag.BackgroundColor | StyleFlag.Color;
 		}
 		
-		protected override void DrawIcon (Cairo.Context cr, int x, int y, int size)
+		protected override void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
-			Cairo.Color color1 = Style.ToCairoColor (editor.ColorStyle.CurrentDebugLineMarkerColor1);
-			Cairo.Color color2 = Style.ToCairoColor (editor.ColorStyle.CurrentDebugLineMarkerColor2);
-			Cairo.Color border = Style.ToCairoColor (editor.ColorStyle.CurrentDebugLineMarkerBorder);
+			Cairo.Color color1 = editor.ColorStyle.CurrentDebugLineMarkerColor1;
+			Cairo.Color color2 = editor.ColorStyle.CurrentDebugLineMarkerColor2;
+			Cairo.Color border = editor.ColorStyle.CurrentDebugLineMarkerBorder;
 		
 			DrawArrow (cr, x, y, size);
 			FillGradient (cr, color1, color2, x, y, size);
@@ -221,12 +224,12 @@ namespace MonoDevelop.Debugger
 	
 	public class DebugStackLineTextMarker: DebugTextMarker
 	{
-		public override Color BackgroundColor {
+		public override Cairo.Color BackgroundColor {
 			get { return editor.ColorStyle.DebugStackLineBg; }
 			set {  }
 		}
 		
-		public override Color Color {
+		public override Cairo.Color Color {
 			get { return editor.ColorStyle.DebugStackLineFg;  }
 			set {  }
 		}
@@ -236,11 +239,11 @@ namespace MonoDevelop.Debugger
 			IncludedStyles |= StyleFlag.BackgroundColor | StyleFlag.Color;
 		}
 		
-		protected override void DrawIcon (Cairo.Context cr, int x, int y, int size)
+		protected override void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
-			Cairo.Color color1 = Style.ToCairoColor (editor.ColorStyle.DebugStackLineMarkerColor1);
-			Cairo.Color color2 = Style.ToCairoColor (editor.ColorStyle.DebugStackLineMarkerColor2);
-			Cairo.Color border = Style.ToCairoColor (editor.ColorStyle.DebugStackLineMarkerBorder);
+			Cairo.Color color1 = editor.ColorStyle.DebugStackLineMarkerColor1;
+			Cairo.Color color2 = editor.ColorStyle.DebugStackLineMarkerColor2;
+			Cairo.Color border = editor.ColorStyle.DebugStackLineMarkerBorder;
 		
 			DrawArrow (cr, x, y, size);
 			FillGradient (cr, color1, color2, x, y, size);
@@ -250,7 +253,7 @@ namespace MonoDevelop.Debugger
 	
 	public class InvalidBreakpointTextMarker: DebugTextMarker
 	{
-		public override Color BackgroundColor {
+		public override Cairo.Color BackgroundColor {
 			get { return editor.ColorStyle.InvalidBreakpointBg; }
 			set {  }
 		}
@@ -260,11 +263,11 @@ namespace MonoDevelop.Debugger
 			IncludedStyles |= StyleFlag.BackgroundColor;
 		}
 		
-		protected override void DrawIcon (Cairo.Context cr, int x, int y, int size)
+		protected override void DrawIcon (Cairo.Context cr, double x, double y, double size)
 		{
-			Cairo.Color color1 = Style.ToCairoColor (editor.ColorStyle.InvalidBreakpointMarkerColor1);
+			Cairo.Color color1 = editor.ColorStyle.InvalidBreakpointMarkerColor1;
 			Cairo.Color color2 = color1;
-			Cairo.Color border = Style.ToCairoColor (editor.ColorStyle.InvalidBreakpointMarkerBorder);
+			Cairo.Color border = editor.ColorStyle.InvalidBreakpointMarkerBorder;
 			DrawCircle (cr, x, y, size);
 			FillGradient (cr, color1, color2, x, y, size);
 			DrawBorder (cr, border, x, y, size);
