@@ -30,10 +30,23 @@ using System;
 using System.Text.RegularExpressions;
 using Mono.TextEditor;
 using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.SourceEditor
 {
-	
+	public class NewIdeViMode : Mono.TextEditor.Vi.NewViEditMode
+	{
+		public NewIdeViMode (ExtensibleTextEditor editor)
+		{
+			this.editor = editor;
+		}
+		
+		protected override void HandleKeypress (Gdk.Key key, uint unicodeKey, Gdk.ModifierType modifier)
+		{
+			base.HandleKeypress (key, unicodeKey, modifier);
+			IdeApp.Workbench.StatusBar.ShowMessage (ViEditor.Message);
+		}
+	}
 	
 	public class IdeViMode : Mono.TextEditor.Vi.ViEditMode
 	{
@@ -50,7 +63,8 @@ namespace MonoDevelop.SourceEditor
 			get { return base.Status; }
 			protected set {
 				base.Status = value;
-				IdeApp.Workbench.StatusBar.ShowMessage (value);
+				// For the (recording) display.
+				IdeApp.Workbench.StatusBar.ShowMessage (base.Status);
 			}
 		}
 		
