@@ -29,21 +29,10 @@ using Mono.TextEditor;
 using MonoDevelop.Core;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Ide;
+using MonoDevelop.Ide.Fonts;
 
 namespace MonoDevelop.SourceEditor
 {
-	public enum EditorFontType {
-		/// <summary>
-		/// Default Monospace font as set in the user's GNOME font properties
-		/// </summary>
-		DefaultMonospace,
-		
-		/// <summary>
-		/// Custom font, will need to get the FontName property for more specifics
-		/// </summary>
-		UserSpecified
-	}
-
 	public enum ControlLeftRightMode {
 		MonoDevelop,
 		Emacs,
@@ -88,11 +77,22 @@ namespace MonoDevelop.SourceEditor
 			LoadAllPrefs ();
 			UpdateStylePolicy (currentPolicy);
 			PropertyService.PropertyChanged += UpdatePreferences;
+			FontService.RegisterFontChangedCallback ("Editor", UpdateFont);
+			FontService.RegisterFontChangedCallback ("MessageBubbles", UpdateFont);
+			
 		}
 		
 		public override void Dispose()
 		{
 			PropertyService.PropertyChanged -= UpdatePreferences;
+			FontService.RemoveCallback (UpdateFont);
+		}
+		
+		void UpdateFont ()
+		{
+			base.FontName = FontName;
+			this.OnChanged (EventArgs.Empty);
+		
 		}
 		
 		void UpdateStylePolicy (MonoDevelop.Ide.Gui.Content.TextStylePolicy currentPolicy)
@@ -128,9 +128,6 @@ namespace MonoDevelop.SourceEditor
 			case "EnableParameterInsight":
 				this.EnableParameterInsight = (bool) args.NewValue;
 				break;
-			case "EnableQuickFinder":
-				this.EnableQuickFinder = (bool) args.NewValue;
-				break;
 			case "UnderlineErrors":
 				this.UnderlineErrors = (bool) args.NewValue;
 				break;
@@ -143,9 +140,6 @@ namespace MonoDevelop.SourceEditor
 				} else 
 					this.IndentStyle = (MonoDevelop.Ide.Gui.Content.IndentStyle) args.NewValue;
 				break;
-			case "EditorFontType":
-				this.EditorFontType = (MonoDevelop.SourceEditor.EditorFontType) args.NewValue;
-				break;
 			case "ShowLineNumberMargin":
 				base.ShowLineNumberMargin = (bool) args.NewValue;
 				break;
@@ -221,10 +215,8 @@ namespace MonoDevelop.SourceEditor
 			this.smartSemicolonPlacement = PropertyService.Get ("SmartSemicolonPlacement", false);
 			this.enableCodeCompletion = PropertyService.Get ("EnableCodeCompletion", true);
 			this.enableParameterInsight = PropertyService.Get ("EnableParameterInsight", true);
-			this.enableQuickFinder = PropertyService.Get ("EnableQuickFinder", true);
 			this.underlineErrors = PropertyService.Get ("UnderlineErrors", true);
 			this.indentStyle = PropertyService.Get ("IndentStyle", MonoDevelop.Ide.Gui.Content.IndentStyle.Smart);
-			this.editorFontType = PropertyService.Get ("EditorFontType", MonoDevelop.SourceEditor.EditorFontType.DefaultMonospace);
 			base.ShowLineNumberMargin = PropertyService.Get ("ShowLineNumberMargin", true);
 			base.ShowFoldMargin = PropertyService.Get ("ShowFoldMargin", true);
 			base.ShowInvalidLines = PropertyService.Get ("ShowInvalidLines", true);
@@ -400,20 +392,6 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 		
-		bool enableQuickFinder;
-		public bool EnableQuickFinder {
-			get {
-				return enableQuickFinder;
-			}
-			set {
-				if (value != this.enableQuickFinder) {
-					this.enableQuickFinder = value;
-					PropertyService.Set ("EnableQuickFinder", value);
-					OnChanged (EventArgs.Empty);
-				}
-			}
-		}
-		
 		bool underlineErrors;
 		public bool UnderlineErrors {
 			get {
@@ -674,19 +652,10 @@ namespace MonoDevelop.SourceEditor
 		
 		public override string FontName {
 			get {
-				if (EditorFontType == EditorFontType.DefaultMonospace) {
-					string font = DesktopService.DefaultMonospaceFont;
-					if (String.IsNullOrEmpty (font))
-						return DEFAULT_FONT;
-					else
-						return font;
-				}
-				return base.FontName;
+				return FontService.FilterFontName (FontService.GetUnderlyingFontName ("Editor"));
 			}
 			set {
-				string newName = !String.IsNullOrEmpty (value) ? value : DEFAULT_FONT;
-				PropertyService.Set ("FontName", newName);
-				base.FontName = newName;
+				throw new InvalidOperationException ("Set font through font service");
 			}
 		}
 		
