@@ -40,6 +40,7 @@ using MonoDevelop.Ide.Gui.Components;
 using MonoDevelop.Ide.Execution;
 using MonoDevelop.Components.Docking;
 using MonoDevelop.Ide;
+using MonoDevelop.Projects;
 
 namespace MonoDevelop.NUnit
 {
@@ -409,9 +410,10 @@ namespace MonoDevelop.NUnit
 		protected void OnUpdateRunTest (CommandArrayInfo info)
 		{
 			UnitTest test = GetSelectedTest ();
-			if (test != null && IdeApp.ProjectOperations.CurrentSelectedProject != null) {
+			if (test != null) {
+				SolutionEntityItem item = test.OwnerObject as SolutionEntityItem;
 				ExecutionModeCommandService.GenerateExecutionModeCommands (
-				    IdeApp.ProjectOperations.CurrentSelectedProject,
+				    item,
 				    test.CanRun,
 				    info);
 			}
