@@ -1,11 +1,9 @@
 using System;
-using System.Collections;
 using System.Threading;
-
 using Gtk;
 
 using MonoDevelop.Core;
-using MonoDevelop.Ide.Gui;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.VersionControl
 {
@@ -25,33 +23,42 @@ namespace MonoDevelop.VersionControl
 		{
 		}
 
-		protected Task() {
+		protected Task()
+		{
 			threadnotify = new ThreadNotify(new ReadyEvent(Wakeup));
-			
-			tracker = IdeApp.Workbench.ProgressMonitors.GetOutputProgressMonitor("Version Control", null, true, true);
 		}
 		
-		protected IProgressMonitor GetProgressMonitor ()
+		protected IProgressMonitor Monitor {
+			get { return tracker; }
+		}
+		
+		protected virtual IProgressMonitor CreateProgressMonitor ()
 		{
-			return tracker;
+			return VersionControlService.GetProgressMonitor (GetDescription ());
 		}
 		
 		public void Start() {
-			tracker.BeginTask(GetDescription(), 0);
-			new Thread(new ThreadStart(BackgroundWorker)) {
-				Name = "VCS background tasks",
-				IsBackground = true,
-			}.Start();
+			tracker = CreateProgressMonitor ();
+			tracker.BeginTask(GetDescription(), 1);
+			ThreadPool.QueueUserWorkItem (BackgroundWorker);
 		}
 		
-		void BackgroundWorker() {
+		void BackgroundWorker (object state)
+		{
 			try {
 				Run();
-				tracker.ReportSuccess(GettextCatalog.GetString ("Done."));
 			} catch (DllNotFoundException e) {
 				tracker.ReportError("The operation could not be completed because a shared library is missing: " + e.Message, null);
 			} catch (Exception e) {
-				tracker.ReportError(e.Message, null);
+				string msg = GettextCatalog.GetString ("Version control operation failed: ");
+				msg += e.Message;
+				if (e.InnerException != null && e.InnerException.Message != e.Message) {
+					msg = msg.Trim ();
+					if (!msg.EndsWith ("."))
+						msg += ".";
+					msg += " " + e.InnerException.Message;
+				}
+				tracker.ReportError (msg, e);
 				Console.Error.WriteLine(e);
 			} finally {			
 				threadnotify.WakeupMain();
