@@ -46,6 +46,8 @@ namespace MonoDevelop.WelcomePage
 		WelcomePageWidget widget;
 		ScrolledWindow scroller;
 		
+		EventHandler newsUpdatedHandler;
+		
 		// netNewsXml is where online the news.xml file can be found
 		static string netNewsXml {
 			get {
@@ -75,9 +77,10 @@ namespace MonoDevelop.WelcomePage
 			this.ContentName = GettextCatalog.GetString ("Welcome");
 			this.IsViewOnly = true;
 			
-			recentChangesHandler = (EventHandler) DispatchService.GuiDispatch (new EventHandler (RecentChangesHandler));
-			IdeApp.Workbench.RecentOpen.RecentProjectChanged += recentChangesHandler;
-			NewsUpdated += (EventHandler) DispatchService.GuiDispatch (new EventHandler (HandleNewsUpdate));
+			recentChangesHandler = DispatchService.GuiDispatch (new EventHandler (RecentChangesHandler));
+			DesktopService.RecentFiles.Changed += recentChangesHandler;
+			newsUpdatedHandler = (EventHandler) DispatchService.GuiDispatch (new EventHandler (HandleNewsUpdate));
+			NewsUpdated += newsUpdatedHandler;
 			
 			UpdateNews ();
 			
@@ -94,19 +97,37 @@ namespace MonoDevelop.WelcomePage
 			scroller.Show ();
 		}
 		
+		static string NewsFile {
+			get {
+				return PropertyService.Locations.Cache.Combine ("WelcomePageNews.xml");
+			}
+		}
+		
+		static XmlReaderSettings GetSafeReaderSettings ()
+		{
+			//allow DTD but not try to resolve it from web
+			return new XmlReaderSettings () {
+				CloseInput = true,
+				ProhibitDtd = false,
+				XmlResolver = null,
+			};
+		}
+		
 		public XmlDocument GetUpdatedXmlDocument ()
 		{
-			string localCachedNewsFile = System.IO.Path.Combine (PropertyService.ConfigPath, "news.xml");
+			string localCachedNewsFile = NewsFile;
+			var settings = GetSafeReaderSettings ();
 			
-			Stream stream = Assembly.GetExecutingAssembly ().GetManifestResourceStream ("WelcomePageContent.xml");
-			XmlDocument contentDoc = new XmlDocument ();
-			using (XmlTextReader reader = new XmlTextReader (stream))
+			var stream = Assembly.GetExecutingAssembly ().GetManifestResourceStream ("WelcomePageContent.xml");
+			var contentDoc = new XmlDocument ();
+			using (var reader = XmlReader.Create (stream, settings))
 				contentDoc.Load (reader);
 			
 			try {
 				if (File.Exists (localCachedNewsFile)) {
-					XmlDocument updateDoc = new XmlDocument (); 
-					using (XmlTextReader reader = new XmlTextReader (localCachedNewsFile))
+					var updateDoc = new XmlDocument ();
+
+					using (var reader = XmlReader.Create (localCachedNewsFile, settings))
 						updateDoc.Load (reader);
 					
 					XmlNode oNodeWhereInsert =
@@ -115,7 +136,9 @@ namespace MonoDevelop.WelcomePage
 						oNodeWhereInsert.AppendChild (contentDoc.ImportNode (link,true)); 
 				}
 			} catch (Exception ex) {
-				LoggingService.LogError ("Error updating Welcome Page content.", ex);
+				LoggingService.LogError ("Error reading Welcome Page content file.", ex);
+				if (File.Exists (localCachedNewsFile))
+					File.Delete (localCachedNewsFile);
 			}
 			
 			return contentDoc;
@@ -123,61 +146,77 @@ namespace MonoDevelop.WelcomePage
 		
 		static void UpdateNewsXmlAsync ()
 		{
-			//check to see if the online news file has been modified since it was last downloaded
 			string netNewsXml = WelcomePageView.netNewsXml;
 			LoggingService.LogInfo ("Updating Welcome Page from '{0}'.", netNewsXml);
 			
-			HttpWebRequest request = (HttpWebRequest) WebRequest.Create (netNewsXml);
-			string localCachedNewsFile = System.IO.Path.Combine (PropertyService.ConfigPath, "news.xml");
-			FileInfo localNewsXml = new FileInfo (localCachedNewsFile);
-			if (localNewsXml.Exists)
-				request.IfModifiedSince = localNewsXml.LastWriteTime;
-			
+			//HACK: .NET blocks on DNS in BeginGetResponse, so use a threadpool thread
+			// see http://stackoverflow.com/questions/1232139#1232930
+			System.Threading.ThreadPool.QueueUserWorkItem ((state) => {
+				var request = (HttpWebRequest) WebRequest.Create (netNewsXml);
+				
+				try {
+					//check to see if the online news file has been modified since it was last downloaded
+					string localCachedNewsFile = NewsFile;
+					var localNewsXml = new FileInfo (localCachedNewsFile);
+					if (localNewsXml.Exists)
+						request.IfModifiedSince = localNewsXml.LastWriteTime;
+					
+					request.BeginGetResponse (HandleResponse, request);
+				} catch (Exception ex) {
+					LoggingService.LogWarning ("Welcome Page news file could not be downloaded.", ex);
+					lock (updateLock)
+						isUpdating = false;
+				}
+			});
+		}
+
+		static void HandleResponse (IAsyncResult ar)
+		{
+			string localCachedNewsFile = NewsFile;
 			try {
-				request.BeginGetResponse (delegate (IAsyncResult ar) {
-					try {
-						var response = (HttpWebResponse) request.EndGetResponse (ar);
-						if (response.StatusCode == HttpStatusCode.OK) {
-							using (var fs = File.Create (localCachedNewsFile))
-								CopyStream (fs, response.GetResponseStream (), response.ContentLength);
-						}
-						NewsUpdated (null, EventArgs.Empty);
-					} catch (System.Net.WebException wex) {
-						var httpResp = wex.Response as HttpWebResponse;
-						if (httpResp != null && httpResp.StatusCode == HttpStatusCode.NotModified) {
-							LoggingService.LogInfo ("Welcome Page already up-to-date.");
-						} else if (httpResp != null && httpResp.StatusCode == HttpStatusCode.NotFound) {
-							LoggingService.LogInfo ("Welcome Page update file not found.", netNewsXml);
-						} else {
-							LoggingService.LogWarning ("Welcome Page news file could not be downloaded.", wex);
-						}
-					} catch (Exception ex) {
-						LoggingService.LogWarning ("Welcome Page news file could not be downloaded.", ex);
-					} finally {
-						lock (updateLock)
-							isUpdating = false;
-					}
-				}, null);
+				var request = (HttpWebRequest) ar.AsyncState;
+				var tempFile = localCachedNewsFile + ".temp";
+				//FIXME: limit this size in case open wifi hotspots provide bad data
+				var response = (HttpWebResponse) request.EndGetResponse (ar);
+				if (response.StatusCode == HttpStatusCode.OK) {
+					using (var fs = File.Create (tempFile))
+						response.GetResponseStream ().CopyTo (fs, 2048);
+				}
+				
+				//check the document is valid, might get bad ones from wifi hotspots etc
+				try {
+					var updateDoc = new XmlDocument ();
+					using (var reader = XmlReader.Create (tempFile, GetSafeReaderSettings ()))
+						updateDoc.Load (reader);
+					updateDoc.SelectSingleNode ("/links");
+				} catch (Exception ex) {
+					LoggingService.LogWarning ("Welcome Page news file is bad, keeping old version.", ex);
+					File.Delete (tempFile);
+					return;
+				}
+				
+				if (File.Exists (localCachedNewsFile))
+					File.Delete (localCachedNewsFile);
+				File.Move (tempFile, localCachedNewsFile);
+				
+				NewsUpdated (null, EventArgs.Empty);
+			} catch (System.Net.WebException wex) {
+				var httpResp = wex.Response as HttpWebResponse;
+				if (httpResp != null && httpResp.StatusCode == HttpStatusCode.NotModified) {
+					LoggingService.LogInfo ("Welcome Page already up-to-date.");
+				} else if (httpResp != null && httpResp.StatusCode == HttpStatusCode.NotFound) {
+					LoggingService.LogInfo ("Welcome Page update file not found.", netNewsXml);
+				} else {
+					LoggingService.LogWarning ("Welcome Page news file could not be downloaded.", wex);
+				}
 			} catch (Exception ex) {
 				LoggingService.LogWarning ("Welcome Page news file could not be downloaded.", ex);
+			} finally {
 				lock (updateLock)
 					isUpdating = false;
 			}
 		}
 		
-		static void CopyStream (Stream fr, Stream to, long remaining)
-		{
-			int position = 0;
-			int readBytes = -1;
-			byte[] buffer = new byte[2048];
-			while (readBytes != 0) {							
-				readBytes = fr.Read (buffer, position, (int) (remaining > 2048 ? 2048 : remaining));
-				position += readBytes;
-				remaining -= readBytes;
-				to.Write (buffer, 0, readBytes);
-			}
-		}
-		
 		public static void UpdateNews ()
 		{
 			if (!WelcomePageOptions.UpdateFromInternet)
@@ -284,7 +323,12 @@ namespace MonoDevelop.WelcomePage
 
 		public override void Dispose ()
 		{
-			IdeApp.Workbench.RecentOpen.RecentProjectChanged -= recentChangesHandler;
+			NewsUpdated -= newsUpdatedHandler;
+			if (recentChangesHandler != null) {
+				DesktopService.RecentFiles.Changed -= recentChangesHandler;
+				recentChangesHandler = null;
+			}
+			base.Dispose ();
 		}
 		
 		public static string TimeSinceEdited (DateTime prjtime)
@@ -301,16 +345,9 @@ namespace MonoDevelop.WelcomePage
 			return GettextCatalog.GetString ("Less than a minute");
 		}
 		
-		public IEnumerable<RecentItem> RecentProjects {
-			get {
-				return IdeApp.Workbench.RecentOpen.RecentProjects;
-			}
-		}
-		
-		public int RecentProjectsCount {
-			get {
-				return IdeApp.Workbench.RecentOpen.RecentProjectsCount;
-			}
+		public IList<RecentFile> GetRecentProjects ()
+		{
+			return DesktopService.RecentFiles.GetProjects ();
 		}
 	}
 }
