@@ -6,13 +6,15 @@ using System.Text.RegularExpressions;
 using System.Xml;
 using System.Xml.Serialization;
 
-using MonoDevelop.Prj2Make.Schema.Prjx;
 using MonoDevelop.Prj2Make.Schema.Csproj;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Core;
+using MonoDevelop.Ide.Gui;
+
+using MonoDevelop.Core.Assemblies;
+using MonoDevelop.CSharp.Project;
 
-using CSharpBinding;
 
 namespace MonoDevelop.Prj2Make
 {
@@ -22,12 +24,13 @@ namespace MonoDevelop.Prj2Make
 		Hashtable projNameInfo = new Hashtable();
 		Hashtable projGuidInfo = new Hashtable();
 		private string prjxFileName;
-		private string cmbxFileName;
 		private string m_strSlnVer;
 		private string m_strCsprojVer;
 		private bool m_bIsUnix;
 		private bool m_bIsMcs;
 		private bool m_bIsUsingLib;
+		
+		TargetFramework fx = Runtime.SystemAssemblyService.GetTargetFramework (TargetFrameworkMoniker.NET_1_1);
  
 		// Flag use to determine if the LIB variable will be used in
 		// the Makefile that prj2make generates
@@ -70,13 +73,6 @@ namespace MonoDevelop.Prj2Make
 			get { return prjxFileName; }
 		}
 
-		// Shuld contain the file name 
-		// of the most resent cmbx generation
-		public string CmbxFileName {
-			get { return cmbxFileName; }
-		}
-
-
 		// Default constructor
 		public SlnMaker()
 		{
@@ -95,7 +91,7 @@ namespace MonoDevelop.Prj2Make
 			StreamReader reader = new StreamReader(fis);
 			strInput = reader.ReadLine();
 
-			match = SlnFileFormat.SlnVersionRegex.Match(strInput);
+			match = SlnVersionRegex.Match(strInput);
 			if (match.Success)
 			{
 				strVersion = match.Groups[1].Value;
@@ -143,7 +139,7 @@ namespace MonoDevelop.Prj2Make
 					string s = reader.ReadLine();
 					Match match;
 
-					match = SlnFileFormat.ProjectRegex.Match(s);
+					match = ProjectRegex.Match(s);
 					if (match.Success)
 					{
 						string projectName = match.Groups[2].Value;
@@ -174,295 +170,18 @@ namespace MonoDevelop.Prj2Make
 			}
 		}
     
-		public string MsSlnHelper(bool isUnixMode, bool isMcsMode, bool isSln, string slnFile, IProgressMonitor monitor)
-		{
-			bool noCommonTargets = false;
-			bool noProjectTargets = false;
-			bool noFlags = false;
-			StringBuilder MakefileBuilder = new StringBuilder();
-    
-			m_bIsUnix = isUnixMode;
-			m_bIsMcs = isMcsMode;
-			
-			if(m_bIsUnix == true && m_bIsMcs == true)
-			{
-				m_bIsUsingLib = true;
-			}
-
-			if (m_bIsUnix)
-			{
-				slash = "/";
-			}
-			else
-			{
-				slash = "\\";
-			}
-    		
-    		string origDir = Directory.GetCurrentDirectory();
-			try
-			{
-				string d = Path.GetDirectoryName(slnFile);
-				if (d != "")
-					Directory.SetCurrentDirectory(d);
-
-				if (isSln == true) 
-				{
-					// Get the sln file version
-					m_strSlnVer = GetSlnFileVersion(slnFile);
-
-					// We invoke the ParseSolution 
-					// by passing the file obtained
-					ParseSolution (slnFile, monitor);
-				} 
-				else 
-				{
-					// Get the Csproj version
-					m_strCsprojVer = GetCsprojFileVersion(slnFile);
-
-					// We invoke the ParseMsCsProj 
-					// by passing the file obtained 
-					ParseMsCsProj (slnFile);
-				}
-    
-				if (!noFlags)
-				{
-					if (m_bIsUnix) // gmake
-					{
-						MakefileBuilder.Append("ifndef TARGET\n");
-						MakefileBuilder.Append("\tTARGET=./bin/Debug\n");        				
-						MakefileBuilder.Append("else\n");
-						MakefileBuilder.Append("\tTARGET=./bin/$(TARGET)\n");
-						MakefileBuilder.Append("endif\n\n");
-           				
-						if (this.m_bIsMcs == false)
-						{
-							MakefileBuilder.Append("MCS=csc\n");
-							MakefileBuilder.Append("MCSFLAGS=-nologo\n\n");
-							MakefileBuilder.Append("ifdef (RELEASE)\n");
-							MakefileBuilder.Append("\tMCSFLAGS=$(MCSFLAGS) -optimize+ -d:TRACE\n");
-							MakefileBuilder.Append("else\n");
-							MakefileBuilder.Append("\tMCSFLAGS=$(MCSFLAGS) -debug+ -d:TRACE,DEBUG\n");
-							MakefileBuilder.Append("endif\n");
-						}
-						else
-						{
-							MakefileBuilder.Append("MCS=mcs\n");
-							MakefileBuilder.Append("ifndef (RELEASE)\n");
-							MakefileBuilder.Append("\tMCSFLAGS=-debug --stacktrace\n");
-							MakefileBuilder.Append("endif\n");
-							// Define and add the information used in the -lib: arguments passed to the
-							// compiler to assist in finding non-fullyqualified assembly references.
-							if(m_bIsMcs == true)
-							{
-								string strlibDir = PkgConfigInvoker.GetPkgVariableValue("mono", "libdir");
-
-								if (strlibDir != null)
-								{
-			    					MakefileBuilder.AppendFormat("LIBS=-lib:{0} -lib:{1}\n\n", 
-										Path.Combine(strlibDir.TrimEnd(), "mono/1.0"),
-										Path.Combine(strlibDir.TrimEnd(), "mono/gtk-sharp")
-									);
-								}							
-							}
-						}        		
-					}
-					else // nmake
-					{
-						MakefileBuilder.Append("!if !defined (TARGET)\n");
-						MakefileBuilder.Append("TARGET=.\\bin\\Debug\n");        				
-						MakefileBuilder.Append("!else\n");
-						MakefileBuilder.Append("TARGET=.\\bin\\$(TARGET)\n");
-						MakefileBuilder.Append("!endif\n\n");
-           				
-						if (m_bIsMcs == false)
-						{
-							MakefileBuilder.Append("MCS=csc\n");
-							MakefileBuilder.Append("MCSFLAGS=-nologo\n\n");
-							MakefileBuilder.Append("!if !defined(RELEASE)\n");
-							MakefileBuilder.Append("MCSFLAGS=$(MCSFLAGS) -optimize+ -d:TRACE\n");
-							MakefileBuilder.Append("!else\n");
-							MakefileBuilder.Append("MCSFLAGS=$(MCSFLAGS) -debug+ -d:TRACE,DEBUG\n");
-							MakefileBuilder.Append("!endif\n");
-						}
-						else
-						{
-							MakefileBuilder.Append("MCS=mcs\n");
-							MakefileBuilder.Append("!if !defined(RELEASE)\n");
-							MakefileBuilder.Append("MCSFLAGS=-debug --stacktrace\n");
-							MakefileBuilder.Append("!endif\n");
-						}    				
-					}
-    
-					MakefileBuilder.Append("\n");
-				}
-				else
-				{
-					MakefileBuilder.Append("!if !defined(MCS)\n");
-					MakefileBuilder.Append("!error You must provide MCS when making\n");
-					MakefileBuilder.Append("!endif\n\n");
-				}
-    
-				foreach (CsprojInfo pi in projNameInfo.Values)
-				{
-					MakefileBuilder.AppendFormat("{0}=$(TARGET){1}{2}\n", pi.makename_ext, slash, pi.assembly_name);
-					MakefileBuilder.AppendFormat("{0}_PDB=$(TARGET){1}{2}\n", pi.makename, slash, pi.assembly_name.Replace(".dll",".pdb"));
-					MakefileBuilder.AppendFormat("{0}_SRC={1}\n", pi.makename, pi.src);
-					MakefileBuilder.AppendFormat("{0}_RES={1}\n\n", pi.makename, pi.res);
-				}
-    
-				foreach (CsprojInfo pi in projNameInfo.Values)
-				{
-					string refs = "";
-					string deps = "";
-    					
-					foreach (MonoDevelop.Prj2Make.Schema.Csproj.Reference rf in pi.Proyecto.CSHARP.Build.References)
-					{
-						if(rf.Package == null || rf.Package.CompareTo("") == 0)
-						{
-							// Add space in between references as
-							// it becomes necessary
-							if (refs != "")
-								refs += " ";
-
-							string assemblyName = rf.AssemblyName;
-
-							// HACK - under Unix filenames are case sensitive
-							// Under Windows there's no agreement on Xml vs XML ;-)    					
-							if (0 == String.Compare(assemblyName, "System.Xml", true))
-							{
-								assemblyName = "System.Xml";
-							}
-							refs += "-r:" + assemblyName + ".dll";
-						}
-						else
-						{
-							try
-							{
-								CsprojInfo pi2 = (CsprojInfo)projGuidInfo[rf.Project];
-
-								if (refs != "")
-									refs += " ";
-
-								if (deps != "")
-									deps += " ";
-
-								refs += "-r:$(" + pi2.makename_ext + ")";
-								deps += "$(" + pi2.makename_ext + ")";
-							}
-							catch(System.NullReferenceException)
-							{
-								refs += String.Format("-r:{0}.dll", rf.Name);
-								deps += String.Format("# Missing dependency project {1} ID:{0}?", rf.Project, 
-									rf.Name);
-								Console.WriteLine(String.Format(
-									"Warning: The project {0}, ID: {1} may be required and appears missing.",
-									rf.Name, rf.Project)
-									);
-							}
-						}
-					}
-    
-					MakefileBuilder.AppendFormat("$({0}): $({1}_SRC) {2}\n", pi.makename_ext, pi.makename, deps);
-    		
-					if (isUnixMode)
-					{
-						MakefileBuilder.Append("\t-mkdir -p $(TARGET)\n");
-					}
-					else
-					{
-						MakefileBuilder.Append("\t-md $(TARGET)\n");
-					}
-
-					// Test to see if any configuratino has the Allow unsafe blocks on
-					if(pi.AllowUnsafeCode == true ) {
-						MakefileBuilder.Append(" -unsafe");
-					}
-
-					// Test for LIBS usage
-					if(m_bIsUsingLib == true) {
-	    				MakefileBuilder.Append(" $(LIBS)");
-					}
-
-					MakefileBuilder.AppendFormat(" {2}{3} -out:$({0}) $({1}_RES) $({1}_SRC)\n", 
-							pi.makename_ext, pi.makename, refs, pi.switches);
-            								
-					MakefileBuilder.Append("\n");
-				}
-    
-				if (!noCommonTargets)
-				{
-					MakefileBuilder.Append("\n");
-					MakefileBuilder.Append("# common targets\n\n");
-					MakefileBuilder.Append("all:\t");
-    
-					bool first = true;
-    
-					foreach (CsprojInfo pi in projNameInfo.Values)
-					{
-						if (!first)
-						{
-							MakefileBuilder.Append(" \\\n\t");
-						}
-						MakefileBuilder.AppendFormat("$({0})", pi.makename_ext);
-						first = false;
-					}
-					MakefileBuilder.Append("\n\n");
-    
-					MakefileBuilder.Append("clean:\n");
-    
-					foreach (CsprojInfo pi in projNameInfo.Values)
-					{
-						if (isUnixMode)
-						{
-							MakefileBuilder.AppendFormat("\t-rm -f \"$({0})\" 2> /dev/null\n", pi.makename_ext);
-							MakefileBuilder.AppendFormat("\t-rm -f \"$({0}_PDB)\" 2> /dev/null\n", pi.makename);
-						}
-						else
-						{
-							MakefileBuilder.AppendFormat("\t-del \"$({0})\" 2> nul\n", pi.makename_ext);
-							MakefileBuilder.AppendFormat("\t-del \"$({0}_PDB)\" 2> nul\n", pi.makename);
-						}
-					}
-					MakefileBuilder.Append("\n");
-				}
-    
-				if (!noProjectTargets)
-				{
-					MakefileBuilder.Append("\n");
-					MakefileBuilder.Append("# project names as targets\n\n");
-					foreach (CsprojInfo pi in projNameInfo.Values)
-					{
-						MakefileBuilder.AppendFormat("{0}: $({1})\n", pi.name, pi.makename_ext);
-					}
-				}
-    			
-				return MakefileBuilder.ToString();
-			}
-			catch (Exception e)
-			{
-				Console.WriteLine("EXCEPTION: {0}\n", e);
-				return "";
-			}
-			finally
-			{
-				Directory.SetCurrentDirectory(origDir);
-			}
-		}
-		
-		public string CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor)
-		{
-			DotNetProject project = CreatePrjxFromCsproj (csprojFileName, monitor, true);
-			return project.FileName;
-		}
-
-		public DotNetProject CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor, bool save)
+		public SolutionEntityItem CreatePrjxFromCsproj (string csprojFileName, IProgressMonitor monitor)
 		{
 			try {
+				FileFormat format = Services.ProjectService.FileFormats.GetFileFormatForFile (csprojFileName, typeof(SolutionEntityItem));
+				if (format != null && format.Id != "VS2003ProjectFileFormat")
+					return (SolutionEntityItem) Services.ProjectService.ReadSolutionItem (monitor, csprojFileName);
+				
 				MonoDevelop.Prj2Make.Schema.Csproj.VisualStudioProject csprojObj = null;
 				
-				monitor.BeginTask (GettextCatalog.GetString ("Importing project: ") + csprojFileName, 5);
+				monitor.BeginTask (GettextCatalog.GetString ("Importing project: ") + csprojFileName, 4);
 				
-				DotNetProject prjxObj = new DotNetProject ("C#", null, null);
+				DotNetAssemblyProject prjxObj = new DotNetAssemblyProject ("C#", null, null);
 				
 				prjxFileName = String.Format ("{0}.mdp",
 					Path.Combine (Path.GetDirectoryName (csprojFileName),
@@ -484,10 +203,10 @@ namespace MonoDevelop.Prj2Make
 				prjxObj.NewFileSearch = NewFileSearch.None;
 				prjxObj.DefaultNamespace = csprojObj.CSHARP.Build.Settings.RootNamespace;
 
-				GetContents (prjxObj, csprojObj.CSHARP.Files.Include, prjxObj.ProjectFiles, monitor);
+				GetContents (prjxObj, csprojObj.CSHARP.Files.Include, prjxObj.Files, monitor);
 				monitor.Step (1);
 				
-				GetReferences (csprojObj.CSHARP.Build.References, prjxObj.ProjectReferences, monitor);
+				GetReferences (prjxObj, csprojObj.CSHARP.Build.References, prjxObj.References, monitor);
 				monitor.Step (1);
 				
 				prjxObj.Configurations.Clear ();
@@ -502,9 +221,6 @@ namespace MonoDevelop.Prj2Make
 						));
 				}
 				monitor.Step (1);
-				if (save)
-					prjxObj.Save (monitor);
-				monitor.Step (1);
 				return prjxObj;
 
 			} catch (Exception ex) {
@@ -515,19 +231,9 @@ namespace MonoDevelop.Prj2Make
 			}
 		}
 
-		public string MsSlnToCmbxHelper (string slnFileName, IProgressMonitor monitor)
+		public Solution MsSlnToCmbxHelper (string slnFileName, IProgressMonitor monitor)
 		{
-			Combine c = MsSlnToCmbxHelper (slnFileName, monitor, true);
-			return c.FileName;
-		}
-
-		public Combine MsSlnToCmbxHelper (string slnFileName, IProgressMonitor monitor, bool save)
-		{
-			Combine cmbxObj = new Combine();
-			cmbxFileName = String.Format ("{0}.mds",
-				Path.Combine(Path.GetDirectoryName(slnFileName),
-				Path.GetFileNameWithoutExtension(slnFileName))
-				);
+			Solution solution = new Solution();
 			
 			monitor.BeginTask (GettextCatalog.GetString ("Importing solution"), 2);
 			try
@@ -546,19 +252,20 @@ namespace MonoDevelop.Prj2Make
 						monitor.ReportWarning (GettextCatalog.GetString ("Project file not found: ") + pi.csprojpath);
 						continue;
 					}
-					DotNetProject prj = CreatePrjxFromCsproj (mappedPath, monitor, save);
+					SolutionEntityItem prj;
+					if (pi.NeedsConversion)
+						prj = CreatePrjxFromCsproj (mappedPath, monitor);
+					else
+						prj = (DotNetProject) Services.ProjectService.ReadSolutionItem (monitor, mappedPath);
+					
 					if (prj == null)
 						return null;
 
 					monitor.Step (1);
-					if (save) {
-						string prjName = prj.FileName;
-						if (prjName != null)
-							cmbxObj.AddEntry (prjName, monitor);
-						else
-							return null;
-					} else {
-						cmbxObj.Entries.Add (prj);
+					solution.RootFolder.Items.Add (prj);
+					foreach (ItemConfiguration conf in prj.Configurations) {
+						if (!solution.GetConfigurations ().Contains (conf.Id))
+							solution.AddConfiguration (conf.Id, false);
 					}
 					monitor.Step (1);
 				}
@@ -566,12 +273,10 @@ namespace MonoDevelop.Prj2Make
 				monitor.EndTask ();
 				monitor.Step (1);
 
-				cmbxObj.FileName = cmbxFileName;
-				if (save)
-					cmbxObj.Save (cmbxFileName, monitor);
-
+				solution.SetLocation (Path.GetDirectoryName (slnFileName), Path.GetFileNameWithoutExtension(slnFileName));
+				
 				monitor.Step (1);
-				return cmbxObj;
+				return solution;
 			}
 			catch (Exception e)
 			{
@@ -584,7 +289,7 @@ namespace MonoDevelop.Prj2Make
 			}
 		}
 		
-		protected void GetReferences (MonoDevelop.Prj2Make.Schema.Csproj.Reference[] References, ProjectReferenceCollection references, IProgressMonitor monitor)
+		protected void GetReferences (Project project, MonoDevelop.Prj2Make.Schema.Csproj.Reference[] References, ProjectReferenceCollection references, IProgressMonitor monitor)
 		{
 			if (References == null || References.Length == 0)
 				return;
@@ -638,8 +343,16 @@ namespace MonoDevelop.Prj2Make
 						if (rname == "System.XML")
 							rname = "System.Xml";
 						
-						string oref = Runtime.SystemAssemblyService.GetAssemblyFullName (rname);
+						string oref = Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyFullName (rname, fx);
 						if (oref == null) {
+							if (rf.HintPath != null) {
+								string asm = MapPath (project.ItemDirectory, rf.HintPath);
+								if (!System.IO.File.Exists (asm))
+									monitor.ReportWarning (GettextCatalog.GetString ("Referenced assembly not found: ") + asm);
+								ProjectReference aref = new ProjectReference (MonoDevelop.Projects.ReferenceType.Assembly, asm);
+								references.Add (aref);
+								continue;
+							}
 							monitor.ReportWarning (GettextCatalog.GetString ("Assembly reference could not be imported: ") + rf.AssemblyName);
 							continue;
 						}
@@ -696,7 +409,7 @@ namespace MonoDevelop.Prj2Make
 
 			// Test to see if file exist in GAC location
 			if (System.IO.File.Exists (strRefFileName)) {
-				ProjectReference rfOut = new ProjectReference (MonoDevelop.Projects.ReferenceType.Gac, Runtime.SystemAssemblyService.GetAssemblyFullName (strRefFileName));
+				ProjectReference rfOut = new ProjectReference (MonoDevelop.Projects.ReferenceType.Gac, Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyFullName (strRefFileName, fx));
 				rfOut.LocalCopy = true;
 				return rfOut;
 			}
@@ -741,14 +454,13 @@ namespace MonoDevelop.Prj2Make
 						flOut.BuildAction = BuildAction.Compile;
 						break;
 					case MonoDevelop.Prj2Make.Schema.Csproj.FileBuildAction.Content:
-						flOut.BuildAction = BuildAction.Nothing;
+						flOut.BuildAction = BuildAction.Content;
 						break;
 					case MonoDevelop.Prj2Make.Schema.Csproj.FileBuildAction.EmbeddedResource:
-						flOut.BuildAction = BuildAction.EmbedAsResource;
-						flOut.ResourceId = MSBuildProjectServiceExtension.GetDefaultResourceIdInternal (flOut);
+						flOut.BuildAction = BuildAction.EmbeddedResource;
 						break;
 					case MonoDevelop.Prj2Make.Schema.Csproj.FileBuildAction.None:
-						flOut.BuildAction = BuildAction.Nothing;
+						flOut.BuildAction = BuildAction.None;
 						break;				
 				}
 				// DependentUpon is relative to flOut
@@ -757,32 +469,29 @@ namespace MonoDevelop.Prj2Make
 			}
 		}
 
-		protected IConfiguration CreateConfigurationBlock (MonoDevelop.Projects.Project project, Config ConfigBlock, string AssemblyName, string OuputType, IProgressMonitor monitor)
+		protected SolutionItemConfiguration CreateConfigurationBlock (MonoDevelop.Projects.DotNetProject project, Config ConfigBlock, string AssemblyName, string OuputType, IProgressMonitor monitor)
 		{
 			DotNetProjectConfiguration confObj = project.CreateConfiguration (ConfigBlock.Name) as DotNetProjectConfiguration;
 
 			confObj.RunWithWarnings = false;
-			confObj.NetRuntime = NetRuntime.MsNet;
 			confObj.DebugMode = ConfigBlock.DebugSymbols;
-			confObj.CompileTarget = (CompileTarget) Enum.Parse (typeof(CompileTarget), OuputType, true);
+			project.CompileTarget = (CompileTarget) Enum.Parse (typeof(CompileTarget), OuputType, true);
 			
 			string dir = MapPath (project.BaseDirectory, ConfigBlock.OutputPath);
 			if (dir == null) {
-				dir = "bin/" + ConfigBlock.Name;
+				dir = Path.Combine ("bin", ConfigBlock.Name);
 				monitor.ReportWarning (string.Format (GettextCatalog.GetString ("Output directory '{0}' can't be mapped to a local directory. The directory '{1}' will be used instead"), ConfigBlock.OutputPath, dir));
 			}
 			confObj.OutputDirectory = dir;
 			confObj.OutputAssembly = AssemblyName;
 			
 			CSharpCompilerParameters compilerParams = new CSharpCompilerParameters ();
-			compilerParams.CsharpCompiler = CsharpCompiler.Mcs;
 			compilerParams.WarningLevel = ConfigBlock.WarningLevel;
 			compilerParams.NoWarnings = "";
 			compilerParams.Optimize = ConfigBlock.Optimize;
 			compilerParams.DefineSymbols = ConfigBlock.DefineConstants;
 			compilerParams.UnsafeCode = ConfigBlock.AllowUnsafeBlocks; 
 			compilerParams.GenerateOverflowChecks = ConfigBlock.CheckForOverflowUnderflow;
-			compilerParams.MainClass = "";
 			
 			return confObj;
 		}
@@ -846,5 +555,24 @@ namespace MonoDevelop.Prj2Make
 				return Path.GetFullPath (path);
 			}
 		}
+
+		// static regexes
+		static Regex projectRegex = null;
+		internal static Regex ProjectRegex {
+			get {
+				if (projectRegex == null)
+					projectRegex = new Regex(@"Project\(""(\{[^}]*\})""\) = ""(.*)"", ""(.*)"", ""(\{[^{]*\})""");
+				return projectRegex;
+			}
+		}
+
+		static Regex slnVersionRegex = null;
+		internal static Regex SlnVersionRegex {
+			get {
+				if (slnVersionRegex == null)
+					slnVersionRegex = new Regex (@"Microsoft Visual Studio Solution File, Format Version (\d?\d.\d\d)");
+				return slnVersionRegex;
+			}
+		}
 	}   
 }
