@@ -39,20 +39,27 @@ namespace Mono.TextEditor
 	
 	public class ChunkStyle
 	{
-		public virtual Gdk.Color Color {
+		public virtual Cairo.Color CairoColor {
 			get;
 			set;
 		}
-
-		Gdk.Color backColor = Gdk.Color.Zero;
-		bool backColorIsZero = true;
-		public virtual Gdk.Color BackgroundColor {
+		
+		public Gdk.Color Color {
 			get {
-				return backColor;
+				return (HslColor)CairoColor;
 			}
-			set {
-				backColor = value;
-				backColorIsZero = value.Equal (Gdk.Color.Zero);
+		}
+		
+		bool backColorIsZero = true;
+		Cairo.Color cairoBackgroundColor = new Cairo.Color (0, 0, 0, 0);
+		public virtual Cairo.Color CairoBackgroundColor {
+			get { return cairoBackgroundColor; }
+			set { cairoBackgroundColor = value; backColorIsZero = false; }
+		}
+		
+		public Gdk.Color BackgroundColor {
+			get {
+				return (HslColor)CairoBackgroundColor;
 			}
 		}
 		
@@ -92,8 +99,9 @@ namespace Mono.TextEditor
 		
 		public ChunkStyle (ChunkStyle style)
 		{
-			Color                = style.Color;
-			BackgroundColor      = style.BackgroundColor;
+			CairoColor           = style.CairoColor;
+			if (!style.backColorIsZero)
+				CairoBackgroundColor = style.CairoBackgroundColor;
 			ChunkProperties      = style.ChunkProperties;
 		}
 
@@ -109,16 +117,19 @@ namespace Mono.TextEditor
 			return Bold ? Pango.Weight.Bold : Pango.Weight.Normal;
 		}
 		
-		public ChunkStyle () : this (Gdk.Color.Zero, Gdk.Color.Zero, ChunkProperties.None)
+		public ChunkStyle ()
 		{
 		}
 		
-		public ChunkStyle (Gdk.Color color) : this (color, Gdk.Color.Zero, ChunkProperties.None)
+		public ChunkStyle (Gdk.Color color)
 		{
+			this.CairoColor =(HslColor) color;
 		}
 		
-		public ChunkStyle (Gdk.Color color, ChunkProperties chunkProperties) : this (color, Gdk.Color.Zero, chunkProperties)
+		public ChunkStyle (Gdk.Color color, ChunkProperties chunkProperties)
 		{
+			this.CairoColor      = (HslColor)color;
+			this.ChunkProperties = chunkProperties;
 		}
 		
 		public ChunkStyle (Gdk.Color color, Gdk.Color bgColor) : this (color, bgColor, ChunkProperties.None)
@@ -127,25 +138,38 @@ namespace Mono.TextEditor
 		
 		public ChunkStyle (Gdk.Color color, Gdk.Color bgColor, ChunkProperties chunkProperties)
 		{
-			this.Color           = color;
-			this.BackgroundColor = bgColor;
+			this.CairoColor           = (HslColor)color;
+			this.CairoBackgroundColor = (HslColor)bgColor;
+			this.ChunkProperties = chunkProperties;
+		}
+		
+		public ChunkStyle (Cairo.Color color, Cairo.Color bgColor, ChunkProperties chunkProperties)
+		{
+			this.CairoColor           = color;
+			this.CairoBackgroundColor = bgColor;
+			this.ChunkProperties = chunkProperties;
+		}
+		
+		public ChunkStyle (Cairo.Color color, ChunkProperties chunkProperties)
+		{
+			this.CairoColor           = color;
 			this.ChunkProperties = chunkProperties;
 		}
 		
 		public override string ToString ()
 		{
-			return string.Format ("[ChunkStyle: Color={0}, BackgroundColor={1}, TransparentBackround={2}, ChunkProperties={3}, Link={4}]", Color, BackgroundColor, TransparentBackround, ChunkProperties, Link);
+			return string.Format ("[ChunkStyle: Color={0}, BackgroundColor={1}, TransparentBackround={2}, ChunkProperties={3}, Link={4}]", CairoColor, CairoBackgroundColor, TransparentBackround, ChunkProperties, Link);
 		}
 		
 		public override int GetHashCode ()
 		{
-			return Color.GetHashCode () ^ Bold.GetHashCode ();
+			return CairoColor.GetHashCode () ^ Bold.GetHashCode ();
 		}
 
 		public override bool Equals (object o)
 		{
 			ChunkStyle c = o as ChunkStyle;
-			return c != null && Bold == c.Bold && Italic == c.Italic && Color.GetHashCode () == c.Color.GetHashCode ();
+			return c != null && Bold == c.Bold && Italic == c.Italic && CairoColor.GetHashCode () == c.CairoColor.GetHashCode ();
 		}
 		
 		public Gdk.GC CreateBgGC (Gdk.Drawable drawable)
