@@ -29,6 +29,7 @@ using System;
 using System.Collections.Generic;
 using System.Xml;
 using Gdk;
+using System.Globalization;
 
 namespace Mono.TextEditor.Highlighting
 {
@@ -37,9 +38,9 @@ namespace Mono.TextEditor.Highlighting
 		Dictionary<string, ChunkStyle> styleLookupTable = new Dictionary<string, ChunkStyle> (); 
 		Dictionary<string, string> customPalette = new Dictionary<string, string> (); 
 		
-		public Color GetColorFromDefinition (string colorName)
+		public Cairo.Color GetColorFromDefinition (string colorName)
 		{
-			return this.GetChunkStyle (colorName).Color;
+			return this.GetChunkStyle (colorName).CairoColor;
 		}
 		
 		#region Named colors
@@ -66,21 +67,21 @@ namespace Mono.TextEditor.Highlighting
 		}
 
 		public const string LineNumberFgHighlightedString = "linenumber.highlight";
-		public virtual Color LineNumberFgHighlighted {
+		public virtual Cairo.Color LineNumberFgHighlighted {
 			get {
 				return GetColorFromDefinition (LineNumberFgHighlightedString);
 			}
 		}
 
 		public const string IconBarBgString = "iconbar";
-		public virtual Color IconBarBg {
+		public virtual Cairo.Color IconBarBg {
 			get {
 				return GetColorFromDefinition (IconBarBgString);
 			}
 		}
 
 		public const string IconBarSeperatorString = "iconbar.separator";
-		public virtual Color IconBarSeperator {
+		public virtual Cairo.Color IconBarSeperator {
 			get {
 				return GetColorFromDefinition (IconBarSeperatorString);
 			}
@@ -94,21 +95,21 @@ namespace Mono.TextEditor.Highlighting
 		}
 
 		public const string FoldLineHighlightedString = "fold.highlight";
-		public virtual Color FoldLineHighlighted {
+		public virtual Cairo.Color FoldLineHighlighted {
 			get {
 				return GetColorFromDefinition (FoldLineHighlightedString);
 			}
 		}
 		
 		public const string LineChangedBgString = "marker.line.changed";
-		public virtual Color LineChangedBg {
+		public virtual Cairo.Color LineChangedBg {
 			get {
 				return GetColorFromDefinition (LineChangedBgString);
 			}
 		}
 		
 		public const string LineDirtyBgString = "marker.line.dirty";
-		public virtual Color LineDirtyBg {
+		public virtual Cairo.Color LineDirtyBg {
 			get {
 				return GetColorFromDefinition (LineDirtyBgString);
 			}
@@ -120,37 +121,51 @@ namespace Mono.TextEditor.Highlighting
 				return GetChunkStyle (SelectionString);
 			}
 		}
+		
+		public const string InactiveSelectionString = "text.selection.inactive";
+		public virtual ChunkStyle InactiveSelection {
+			get {
+				return GetChunkStyle (InactiveSelectionString);
+			}
+		}
 
 		public const string LineMarkerString = "marker.line";
-		public virtual Color LineMarker {
+		public virtual Cairo.Color LineMarker {
 			get {
 				return GetColorFromDefinition (LineMarkerString);
 			}
 		}
 
 		public const string RulerString = "marker.ruler";
-		public virtual Color Ruler {
+		public virtual Cairo.Color Ruler {
 			get {
 				return GetColorFromDefinition (RulerString);
 			}
 		}
 
 		public const string WhitespaceMarkerString = "marker.whitespace";
-		public virtual Color WhitespaceMarker {
+		public virtual Cairo.Color WhitespaceMarker {
 			get {
 				return GetColorFromDefinition (WhitespaceMarkerString);
 			}
 		}
+		
+		public const string EolWhitespaceMarkerString = "marker.whitespace.eol";
+		public virtual Cairo.Color EolWhitespaceMarker {
+			get {
+				return GetColorFromDefinition (EolWhitespaceMarkerString);
+			}
+		}
 
 		public const string InvalidLineMarkerString = "marker.invalidline";
-		public virtual Color InvalidLineMarker {
+		public virtual Cairo.Color InvalidLineMarker {
 			get {
 				return GetColorFromDefinition (InvalidLineMarkerString);
 			}
 		}
 		
 		public const string FoldToggleMarkerString = "fold.togglemarker";
-		public virtual Color FoldToggleMarker {
+		public virtual Cairo.Color FoldToggleMarker {
 			get {
 				return GetColorFromDefinition (FoldToggleMarkerString);
 			}
@@ -164,165 +179,171 @@ namespace Mono.TextEditor.Highlighting
 		}
 		
 		public const string BookmarkColor2String = "marker.bookmark.color2";
-		public virtual Color BookmarkColor2 {
+		public virtual Cairo.Color BookmarkColor2 {
 			get {
 				return GetColorFromDefinition (BookmarkColor2String);
 			}
 		}
 		
 		public const string BookmarkColor1String = "marker.bookmark.color1";
-		public virtual Color BookmarkColor1 {
+		public virtual Cairo.Color BookmarkColor1 {
 			get {
 				return GetColorFromDefinition (BookmarkColor1String);
 			}
 		}
-
+		
+		public const string ReadOnlyTextBgString = "text.background.readonly";
+		public Cairo.Color ReadOnlyTextBg {
+			get {
+				return GetColorFromDefinition (ReadOnlyTextBgString);
+			}
+		}
 		
 		public const string SearchTextBgString = "text.background.searchresult";
-		public Color SearchTextBg {
+		public Cairo.Color SearchTextBg {
 			get {
 				return GetColorFromDefinition (SearchTextBgString);
 			}
 		}
 		
 		public const string SearchTextMainBgString = "text.background.searchresult-main";
-		public Color SearchTextMainBg {
+		public Cairo.Color SearchTextMainBg {
 			get {
 				return GetColorFromDefinition (SearchTextMainBgString);
 			}
 		}
 
 		public const string BreakpointString = "marker.breakpoint";
-		public Color BreakpointFg {
+		public Cairo.Color BreakpointFg {
 			get {
-				return GetChunkStyle (BreakpointString).Color;
+				return GetChunkStyle (BreakpointString).CairoColor;
 			}
 		}
-		public Color BreakpointBg {
+		public Cairo.Color BreakpointBg {
 			get {
-				return GetChunkStyle (BreakpointString).BackgroundColor;
+				return GetChunkStyle (BreakpointString).CairoBackgroundColor;
 			}
 		}
 
 		public const string BreakpointMarkerColor2String = "marker.breakpoint.color2";
-		public Color BreakpointMarkerColor2 {
+		public Cairo.Color BreakpointMarkerColor2 {
 			get {
 				return GetColorFromDefinition (BreakpointMarkerColor2String);
 			}
 		}
 
 		public const string BreakpointMarkerColor1String = "marker.breakpoint.color1";
-		public Color BreakpointMarkerColor1 {
+		public Cairo.Color BreakpointMarkerColor1 {
 			get {
 				return GetColorFromDefinition (BreakpointMarkerColor1String);
 			}
 		}
 
 		public const string CurrentDebugLineString = "marker.debug.currentline";
-		public Color CurrentDebugLineFg {
+		public Cairo.Color CurrentDebugLineFg {
 			get {
-				return GetChunkStyle (CurrentDebugLineString).Color;
+				return GetChunkStyle (CurrentDebugLineString).CairoColor;
 			}
 		}
 		
-		public Color CurrentDebugLineBg {
+		public Cairo.Color CurrentDebugLineBg {
 			get {
-				return GetChunkStyle (CurrentDebugLineString).BackgroundColor;
+				return GetChunkStyle (CurrentDebugLineString).CairoBackgroundColor;
 			}
 		}
 
 		public const string CurrentDebugLineMarkerColor2String = "marker.debug.currentline.color2";
-		public Color CurrentDebugLineMarkerColor2 {
+		public Cairo.Color CurrentDebugLineMarkerColor2 {
 			get {
 				return GetColorFromDefinition (CurrentDebugLineMarkerColor2String);
 			}
 		}
 
 		public const string CurrentDebugLineMarkerColor1String = "marker.debug.currentline.color1";
-		public Color CurrentDebugLineMarkerColor1 {
+		public Cairo.Color CurrentDebugLineMarkerColor1 {
 			get {
 				return GetColorFromDefinition (CurrentDebugLineMarkerColor1String);
 			}
 		}
 
 		public const string CurrentDebugLineMarkerBorderString = "marker.debug.currentline.border";
-		public Color CurrentDebugLineMarkerBorder {
+		public Cairo.Color CurrentDebugLineMarkerBorder {
 			get {
 				return GetColorFromDefinition (CurrentDebugLineMarkerBorderString);
 			}
 		}
 		
 		public const string DebugStackLineString = "marker.debug.stackline";
-		public Color DebugStackLineFg {
+		public Cairo.Color DebugStackLineFg {
 			get {
-				return GetChunkStyle (DebugStackLineString).Color;
+				return GetChunkStyle (DebugStackLineString).CairoColor;
 			}
 		}
 		
-		public Color DebugStackLineBg {
+		public Cairo.Color DebugStackLineBg {
 			get {
-				return GetChunkStyle (DebugStackLineString).BackgroundColor;
+				return GetChunkStyle (DebugStackLineString).CairoBackgroundColor;
 			}
 		}
 
 		public const string DebugStackLineMarkerColor2String = "marker.debug.stackline.color2";
-		public Color DebugStackLineMarkerColor2 {
+		public Cairo.Color DebugStackLineMarkerColor2 {
 			get {
 				return GetColorFromDefinition (DebugStackLineMarkerColor2String);
 			}
 		}
 
 		public const string DebugStackLineMarkerColor1String = "marker.debug.stackline.color1";
-		public Color DebugStackLineMarkerColor1 {
+		public Cairo.Color DebugStackLineMarkerColor1 {
 			get {
 				return GetColorFromDefinition (DebugStackLineMarkerColor1String);
 			}
 		}
 
 		public const string DebugStackLineMarkerBorderString = "marker.debug.stackline.border";
-		public Color DebugStackLineMarkerBorder {
+		public Cairo.Color DebugStackLineMarkerBorder {
 			get {
 				return GetColorFromDefinition (DebugStackLineMarkerBorderString);
 			}
 		}
 
 		public const string InvalidBreakpointBgString = "marker.breakpoint.invalid.background";
-		public Color InvalidBreakpointBg {
+		public Cairo.Color InvalidBreakpointBg {
 			get {
 				return GetColorFromDefinition (InvalidBreakpointBgString);
 			}
 		}
 
 		public const string InvalidBreakpointMarkerColor1String = "marker.breakpoint.invalid.color1";
-		public Color InvalidBreakpointMarkerColor1 {
+		public Cairo.Color InvalidBreakpointMarkerColor1 {
 			get {
 				return GetColorFromDefinition (InvalidBreakpointMarkerColor1String);
 			}
 		}
 
 		public const string DisabledBreakpointBgString = "marker.breakpoint.disabled.background";
-		public Color DisabledBreakpointBg {
+		public Cairo.Color DisabledBreakpointBg {
 			get {
 				return GetColorFromDefinition (DisabledBreakpointBgString);
 			}
 		}
 
 		public const string InvalidBreakpointMarkerBorderString = "marker.breakpoint.invalid.border";
-		public Color InvalidBreakpointMarkerBorder {
+		public Cairo.Color InvalidBreakpointMarkerBorder {
 			get {
 				return GetColorFromDefinition (InvalidBreakpointMarkerBorderString);
 			}
 		}
 		
 		public const string ErrorUnderlineString = "marker.underline.error";
-		public Color ErrorUnderline {
+		public Cairo.Color ErrorUnderline {
 			get {
 				return GetColorFromDefinition (ErrorUnderlineString);
 			}
 		}
 		
 		public const string WarningUnderlineString = "marker.underline.warning";
-		public Color WarningUnderline {
+		public Cairo.Color WarningUnderline {
 			get {
 				return GetColorFromDefinition (WarningUnderlineString);
 			}
@@ -359,12 +380,12 @@ namespace Mono.TextEditor.Highlighting
 		
 		public string Name {
 			get;
-			private set;
+			set;
 		}
 		
 		public string Description {
 			get;
-			private set;
+			set;
 		}
 		
 		public static Cairo.Color ToCairoColor (Gdk.Color color)
@@ -374,6 +395,13 @@ namespace Mono.TextEditor.Highlighting
 			                        (double)color.Blue / ushort.MaxValue);
 		}
 		
+		public static Gdk.Color ToGdkColor (Cairo.Color color)
+		{
+			return new Gdk.Color ((byte)(color.R  * 255),
+			                        (byte)(color.G * 255),
+			                        (byte)(color.B * 255));
+		}
+				
 		public static Cairo.Color ToCairoColor (Gdk.Color color, double alpha)
 		{
 			return new Cairo.Color ((double)color.Red / ushort.MaxValue,
@@ -385,6 +413,8 @@ namespace Mono.TextEditor.Highlighting
 		protected Style ()
 		{
 			SetStyle (DefaultString, 0, 0, 0, 255, 255, 255);
+			SetStyle (ReadOnlyTextBgString, 0xFA, 0xFA, 0xF8);
+			
 			GetChunkStyle (DefaultString).ChunkProperties |= ChunkProperties.TransparentBackground;
 			
 			SetStyle (CaretString, DefaultString);
@@ -403,6 +433,7 @@ namespace Mono.TextEditor.Highlighting
 			SetStyle (LineChangedBgString, 108, 226, 108);
 			
 			SetStyle (SelectionString,     255, 255, 255, 96, 87, 210);
+			SetStyle (InactiveSelectionString,     255, 255, 255, 196, 196, 196);
 			
 			SetStyle (LineMarkerString,    200, 255, 255);
 			SetStyle (RulerString,         187, 187, 187);
@@ -433,7 +464,7 @@ namespace Mono.TextEditor.Highlighting
 			SetStyle (SearchTextBgString, 255, 226, 185);
 			SetStyle (SearchTextMainBgString, 243, 221, 72);
 			
-			SetStyle (BracketHighlightRectangleString, 128, 128, 128, 196, 196, 196);
+			SetStyle (BracketHighlightRectangleString, 0xd3, 0xd7, 0xcf, 0xee, 0xee, 0xec);
 			
 			SetStyle (BookmarkColor1String, 255, 255, 255);
 			SetStyle (BookmarkColor2String, 105, 156, 235);
@@ -487,6 +518,12 @@ namespace Mono.TextEditor.Highlighting
 			SetStyleFromWeb ("bubble.inactive.error.dark.color2", "#ebe8e6");
 			SetStyleFromWeb ("bubble.inactive.error.line.top", "#f2f2f2");
 			SetStyleFromWeb ("bubble.inactive.error.line.bottom", "#eceaea");
+			
+			//regions in ASP.NET, T4, etc.
+			SetStyle ("template",           "text");
+			SetStyle ("template.tag",       "constant.language");
+			SetStyle ("template.directive", "constant.language");
+			
 		}
 		
 		protected void PopulateDefaults ()
@@ -539,6 +576,10 @@ namespace Mono.TextEditor.Highlighting
 			SetStyle ("keyword.declaration", 165,  42,  42, BOLD);
 			SetStyle ("keyword.parameter",   165,  42,  42, BOLD);
 			SetStyle ("keyword.misc",        165,  42,  42, BOLD);
+			
+			SetStyle ("template.directive", 0x6A, 0x5A, 0xCD, 0xF0, 0xF0, 0xD0);
+			SetStyle ("template.tag",       0x6A, 0x5A, 0xCD, 0xF0, 0xF0, 0xD0);
+			SetStyle ("template",           0x00, 0x00, 0x00, 0xF0, 0xF0, 0xD0);
 		}
 		
 		const ChunkProperties BOLD   = ChunkProperties.Bold;
@@ -586,7 +627,7 @@ namespace Mono.TextEditor.Highlighting
 		{
 			ChunkStyle style;
 			if (!styleLookupTable.TryGetValue (DefaultString, out style)) {
-				style = new ChunkStyle (GetColorFromDefinition (DefaultString));
+				style = new ChunkStyle (ToGdkColor (GetColorFromDefinition (DefaultString)));
 				styleLookupTable[DefaultString] = style;
 			}
 			return style;
@@ -594,6 +635,8 @@ namespace Mono.TextEditor.Highlighting
 		
 		public ChunkStyle GetChunkStyle (string name)
 		{
+			if (name == null)
+				return GetDefaultChunkStyle ();
 			ChunkStyle style;
 			if (styleLookupTable.TryGetValue (name, out style))
 				return style;
@@ -616,8 +659,8 @@ namespace Mono.TextEditor.Highlighting
 		
 		public void SetChunkStyle (string name, string weight, string foreColor, string backColor)
 		{
-			Gdk.Color color   = !String.IsNullOrEmpty (foreColor) ? this.GetColorFromString (foreColor) : Gdk.Color.Zero;
-			Gdk.Color bgColor = !String.IsNullOrEmpty (backColor) ? this.GetColorFromString (backColor) : Gdk.Color.Zero;
+			Cairo.Color color   = !string.IsNullOrEmpty (foreColor) ? this.GetColorFromString (foreColor) : new Cairo.Color (0, 0, 0);
+			Cairo.Color bgColor = !string.IsNullOrEmpty (backColor) ? this.GetColorFromString (backColor) : new Cairo.Color (0, 0, 0);
 			ChunkProperties properties = ChunkProperties.None;
 			if (weight != null) {
 				if (weight.ToUpper ().IndexOf ("BOLD") >= 0)
@@ -627,19 +670,37 @@ namespace Mono.TextEditor.Highlighting
 				if (weight.ToUpper ().IndexOf ("UNDERLINE") >= 0)
 					properties |= ChunkProperties.Underline;
 			}
-			SetStyle (name, new ChunkStyle (color, bgColor, properties));
+			SetStyle (name,  !string.IsNullOrEmpty (backColor) ? new ChunkStyle (color, bgColor, properties) : new ChunkStyle (color, properties));
 		}
 		
-		public Gdk.Color GetColorFromString (string colorString)
+		static int GetNumber (string str, int offset)
 		{
-			if (customPalette.ContainsKey (colorString))
-				return this.GetColorFromString (customPalette[colorString]);
-			if (styleLookupTable.ContainsKey (colorString)) 
-				return styleLookupTable[colorString].Color;
-			Gdk.Color result = new Color ();
-			if (!Gdk.Color.Parse (colorString, ref result)) 
-				throw new Exception ("Can't parse color: " + colorString);
-			return result;
+			return int.Parse (str.Substring (offset, 2), NumberStyles.HexNumber);
+		}
+		
+		public Cairo.Color GetColorFromString (string colorString)
+		{
+			string refColorString;
+			if (customPalette.TryGetValue (colorString, out refColorString))
+				return this.GetColorFromString (refColorString);
+			ChunkStyle style;
+			if (styleLookupTable.TryGetValue (colorString, out style))
+				return style.CairoColor;
+			if (colorString.Length > 0 && colorString[0] == '#') {
+				if (colorString.Length == 9) {
+					// #AARRGGBB
+					return new Cairo.Color ( GetNumber (colorString, 3) / 255.0, GetNumber (colorString, 5) / 255.0, GetNumber (colorString, 7) / 255.0, GetNumber (colorString, 1) / 255.0);
+				}
+				if (colorString.Length == 7) {
+					// #RRGGBB
+					return new Cairo.Color ( GetNumber (colorString, 1) / 255.0, GetNumber (colorString, 3) / 255.0, GetNumber (colorString, 5) / 255.0);
+				}
+				throw new ArgumentException ("colorString", "colorString must either be #RRGGBB (length 7) or #AARRGGBB (length 9) your string " + colorString + " is invalid because it has a length of " + colorString.Length);
+			} 
+			Gdk.Color color = new Gdk.Color ();
+			if (Gdk.Color.Parse (colorString, ref color))
+				return (Cairo.Color)((HslColor)color);
+			throw new Exception ("Failed to parse color or find named color '" + colorString + "'");
 		}
 		
 		public const string NameAttribute = "name";
@@ -648,8 +709,8 @@ namespace Mono.TextEditor.Highlighting
 		{
 			string name    = reader.GetAttribute ("name"); 
 			string weight  = reader.GetAttribute ("weight") ?? curWeight;
-			string color   = reader.GetAttribute ("color") ?? curColor;
-			string bgColor = reader.GetAttribute ("bgColor") ?? curBgColor;
+			string color   = reader.GetAttribute ("color");
+			string bgColor = reader.GetAttribute ("bgColor");
 			string fullName;
 			if (String.IsNullOrEmpty (curName)) {
 				fullName = name;
