@@ -39,25 +39,25 @@ namespace Mono.TextEditor.Highlighting
 	{
 		public static readonly SyntaxMode Default = new SyntaxMode ();
 		protected List<Rule> rules = new List<Rule> ();
-		
+
 		public string MimeType {
 			get;
 			private set;
 		}
-		
+
 		public IEnumerable<Rule> Rules {
 			get {
 				return rules;
 			}
 		}
-		
+
 		public SyntaxMode() : base (null)
 		{
 			DefaultColor = "text";
 			Name = "<root>";
 			this.Delimiter = "&()<>{}[]~!%^*-+=|\\#/:;\"' ,\t.?";
 		}
-		
+
 		public bool Validate (Style style)
 		{
 			if (!GetIsValid (style)) {
@@ -70,12 +70,12 @@ namespace Mono.TextEditor.Highlighting
 			}
 			return true;
 		}
-		
+
 		public virtual Chunk GetChunks (Document doc, Style style, LineSegment line, int offset, int length)
 		{
 			SpanParser spanParser = CreateSpanParser (doc, this, line, null);
 			ChunkParser chunkParser = CreateChunkParser (spanParser, doc, style, this, line);
-			Chunk result = chunkParser.GetChunks (line.Offset, line.EditableLength);
+			Chunk result = chunkParser.GetChunks (chunkParser.lineOffset, line.EditableLength);
 			if (SemanticRules != null) {
 				foreach (SemanticRule sematicRule in SemanticRules) {
 					sematicRule.Analyze (doc, line, result, offset, offset + length);
@@ -92,7 +92,8 @@ namespace Mono.TextEditor.Highlighting
 						result.Length = endOffset - offset;
 					}
 				}
-				if (result != null && offset + length != line.Offset + line.EditableLength) {
+
+				if (result != null && offset + length != chunkParser.lineOffset + line.EditableLength) {
 					// crop to end
 					Chunk cur = result;
 					while (cur != null && cur.EndOffset < offset + length) {
@@ -104,43 +105,43 @@ namespace Mono.TextEditor.Highlighting
 					}
 				}
 			}
-			
+
 			return result;
 		}
-		
+
 		public virtual string GetTextWithoutMarkup (Document doc, Style style, int offset, int length)
 		{
 			return doc.GetTextAt (offset, length);
 		}
-		
+
 		public string GetMarkup (Document doc, ITextEditorOptions options, Style style, int offset, int length, bool removeIndent)
 		{
 			return GetMarkup (doc, options, style, offset, length, removeIndent, true, true);
 		}
-		
+
 		public static string ColorToPangoMarkup (Gdk.Color color)
 		{
 			return string.Format ("#{0:X2}{1:X2}{2:X2}", color.Red >> 8, color.Green >> 8, color.Blue >> 8);
 		}
-		
+
 		public string GetMarkup (Document doc, ITextEditorOptions options, Style style, int offset, int length, bool removeIndent, bool useColors, bool replaceTabs)
 		{
 			int indentLength = GetIndentLength (doc, offset, length, false);
 			int curOffset = offset;
-			
+
 			StringBuilder result = new StringBuilder ();
 			while (curOffset < offset + length && curOffset < doc.Length) {
 				LineSegment line = doc.GetLineByOffset (curOffset);
 				int toOffset = System.Math.Min (line.Offset + line.EditableLength, offset + length);
 				Stack<ChunkStyle> styleStack = new Stack<ChunkStyle> ();
 				for (Chunk chunk = GetChunks (doc, style, line, curOffset, toOffset - curOffset); chunk != null; chunk = chunk.Next) {
-					
+
 					ChunkStyle chunkStyle = chunk.GetChunkStyle (style);
 					bool setBold = chunkStyle.Bold && (styleStack.Count == 0 || !styleStack.Peek ().Bold) ||
 						!chunkStyle.Bold && (styleStack.Count == 0 || styleStack.Peek ().Bold);
 					bool setItalic = chunkStyle.Italic && (styleStack.Count == 0 || !styleStack.Peek ().Italic) ||
 						!chunkStyle.Italic && (styleStack.Count == 0 || styleStack.Peek ().Italic);
-					bool setUnderline = chunkStyle.Underline && (styleStack.Count == 0 || !styleStack.Peek ().Underline) || 
+					bool setUnderline = chunkStyle.Underline && (styleStack.Count == 0 || !styleStack.Peek ().Underline) ||
 						!chunkStyle.Underline && (styleStack.Count == 0 || styleStack.Peek ().Underline);
 					bool setColor = styleStack.Count == 0 || TextViewMargin.GetPixel (styleStack.Peek ().Color) != TextViewMargin.GetPixel (chunkStyle.Color);
 					if (setColor || setBold || setItalic || setUnderline) {
@@ -163,7 +164,7 @@ namespace Mono.TextEditor.Highlighting
 						result.Append(">");
 						styleStack.Push (chunkStyle);
 					}
-					
+
 					for (int i = 0; i < chunk.Length && chunk.Offset + i < doc.Length; i++) {
 						char ch = chunk.GetCharAt (doc, chunk.Offset + i);
 						switch (ch) {
@@ -193,7 +194,7 @@ namespace Mono.TextEditor.Highlighting
 					result.Append("</span>");
 					styleStack.Pop ();
 				}
-				
+
 				curOffset = line.EndOffset;
 				if (removeIndent)
 					curOffset += indentLength;
@@ -202,7 +203,7 @@ namespace Mono.TextEditor.Highlighting
 			}
 			return result.ToString ();
 		}
-		
+
 		public static int GetIndentLength (Document doc, int offset, int length, bool skipFirstLine)
 		{
 			int curOffset = offset;
@@ -218,183 +219,221 @@ namespace Mono.TextEditor.Highlighting
 			}
 			return indentLength == int.MaxValue ? 0 : indentLength;
 		}
-		
-		public virtual SpanParser CreateSpanParser (Document doc, SyntaxMode mode, LineSegment line, Stack<Span> spanStack)
+
+		public virtual SpanParser CreateSpanParser (Document doc, SyntaxMode mode, LineSegment line, CloneableStack<Span> spanStack)
 		{
-			return new SpanParser (doc, mode, line, spanStack);
+			return new SpanParser (doc, mode, spanStack ?? line.StartSpan.Clone ());
 		}
-		
+
 		public virtual ChunkParser CreateChunkParser (SpanParser spanParser, Document doc, Style style, SyntaxMode mode, LineSegment line)
 		{
 			return new ChunkParser (spanParser, doc, style, mode, line);
 		}
-		
+
 		public class SpanParser
 		{
 			protected SyntaxMode mode;
-			protected Stack<Span> spanStack;
+			protected CloneableStack<Span> spanStack;
 			protected Stack<Rule> ruleStack;
-
 			protected Document doc;
-			protected LineSegment line;
-			
 			int maxEnd;
-			
+
 			public Rule CurRule {
-				get {
-					if (ruleStack.Count == 0)
-						return mode;
-					return ruleStack.Peek ();
-				}
-			}
-			
-			public Span CurSpan {
-				get {
-					if (spanStack.Count == 0)
-						return null;
-					return spanStack.Peek ();
-				}
+				get;
+				private set;
 			}
 
-			public Stack<Span> SpanStack {
+			public Span CurSpan {
+				get;
+				private set;
+			}
+
+			public CloneableStack<Span> SpanStack {
 				get {
 					return spanStack;
 				}
+				set {
+					this.spanStack = value;
+				}
 			}
 
 			public Stack<Rule> RuleStack {
 				get {
 					return ruleStack;
 				}
+				set {
+					this.ruleStack = value;
+					this.CurRule = ruleStack.Peek ();
+				}
 			}
-			
-			public SpanParser (Document doc, SyntaxMode mode, LineSegment line, Stack<Span> spanStack)
+
+			Stack<Rule> CreateRuleStack ()
+			{
+				var result = new Stack<Rule> ();
+				if (mode == null)
+					return result;
+				Rule rule = mode;
+				result.Push (mode);
+				foreach (Span span in this.spanStack.Reverse ()) {
+					Rule tmp = rule.GetRule (span.Rule) ?? this.CurRule;
+					result.Push (tmp);
+					rule = tmp;
+				}
+				return result;
+			}
+
+			public SpanParser (Document doc, SyntaxMode mode, CloneableStack<Span> spanStack)
 			{
 				if (doc == null)
 					throw new ArgumentNullException ("doc");
 				this.doc  = doc;
 				this.mode = mode;
-				this.line = line;
-				this.spanStack = spanStack ?? new Stack<Span> (line != null && line.StartSpan != null ? line.StartSpan : new Span[0]);
-				//this.ruleStack = ruleStack ?? new Stack<Span> (line.StartRule != null ? line.StartRule : new Rule[0]);
-				
-				ruleStack = new Stack<Rule> ();
-				if (mode != null) 
-					ruleStack.Push (mode);
-				List<Rule> rules = new List<Rule> ();
-				foreach (Span span in this.spanStack) {
-					Rule rule = CurRule.GetRule (span.Rule);
-					rules.Add (rule ?? CurRule);
-				}
-				for (int i = rules.Count - 1; i >= 0 ; i--) {
-					ruleStack.Push (rules[i]);
-				}
+				this.SpanStack = spanStack;
+				this.CurRule = mode;
+				this.ruleStack = CreateRuleStack ();
+				this.CurRule = ruleStack.Peek ();
+				this.CurSpan = spanStack.Count > 0 ? spanStack.Peek () : null;
+				FoundSpanBegin = DefaultFoundSpanBegin;
+				FoundSpanEnd = DefaultFoundSpanEnd;
+				FoundSpanExit = DefaultFoundSpanEnd;
+				ParseChar = delegate (ref int i, char ch) {};
 			}
-			
+
+			void DefaultFoundSpanBegin (Span span, int offset, int length)
+			{
+				PushSpan (span, GetRule (span));
+			}
+
+			void DefaultFoundSpanEnd (Span span, int offset, int length)
+			{
+				PopSpan ();
+			}
+
+			public void PushSpan (Span span, Rule rule)
+			{
+				spanStack.Push (span);
+				ruleStack.Push (rule);
+				this.CurRule = rule;
+				this.CurSpan = span;
+			}
+
+			public void PopSpan ()
+			{
+				if (spanStack.Count > 0)
+					spanStack.Pop ();
+				if (ruleStack.Count > 1)
+					ruleStack.Pop ();
+				this.CurRule = ruleStack.Peek ();
+				this.CurSpan = spanStack.Count > 0 ? spanStack.Peek () : null;
+			}
+
 			public Rule GetRule (Span span)
 			{
 				if (string.IsNullOrEmpty (span.Rule))
 					return new Rule (mode);
 				return CurRule.GetRule (span.Rule);
 			}
-			
-			public delegate void SpanDelegate (Span span, int offset, int length);
+
 			public delegate void CharParser (ref int i, char ch);
-			
-			public event SpanDelegate FoundSpanBegin;
-			public virtual void OnFoundSpanBegin (Span span, int offset, int length)
-			{
-				if (FoundSpanBegin != null)
-					FoundSpanBegin (span, offset, length);
-			}
-			
-			public event SpanDelegate FoundSpanExit;
-			public virtual void OnFoundSpanExit (Span span, int offset, int length)
-			{
-				if (FoundSpanExit != null)
-					FoundSpanExit (span, offset, length);
-			}
-			
-			public event SpanDelegate FoundSpanEnd;
-			public virtual void OnFoundSpanEnd (Span span, int offset, int length)
-			{
-				if (FoundSpanEnd != null)
-					FoundSpanEnd (span, offset, length);
-			}
-			
-			public event CharParser ParseChar;
-			public void OnParseChar (ref int i, char ch)
-			{
-				if (ParseChar != null)
-					ParseChar (ref i, ch);
-			}
-			
+
+			public Action<Span, int, int> FoundSpanBegin;
+			public Action<Span, int, int> FoundSpanExit;
+			public Action<Span, int, int> FoundSpanEnd;
+
+			public CharParser ParseChar;
+
 			protected virtual void ScanSpan (ref int i)
 			{
+				int textOffset = i - StartOffset;
+				
 				for (int j = 0; j < CurRule.Spans.Length; j++) {
 					Span span = CurRule.Spans[j];
-					
-					if (span.BeginFlags.Contains ("startsLine") && line != null && i != line.Offset)
+
+					if ((span.BeginFlags & SpanBeginFlags.StartsLine) == SpanBeginFlags.StartsLine) {
+						if (textOffset != 0) {
+							char ch = CurText[textOffset - 1];
+							if (ch != '\n'&& ch != '\r')
+								continue;
+						}
+					} 
+
+					RegexMatch match = span.Begin.TryMatch (CurText, textOffset);
+					if (!match.Success)
 						continue;
-					
-					RegexMatch match = span.Begin.TryMatch (doc, i);
-					if (match.Success) {
-						bool mismatch = false;
-						if (span.BeginFlags.Contains ("firstNonWs") && line != null) {
-							for (int k = line.Offset; k < i; k++) {
-								if (!Char.IsWhiteSpace (doc.GetCharAt (k))) {
-									mismatch = true;
-									break;
-								}
+					// scan for span exit which cancels the span.
+					if ((span.ExitFlags & SpanExitFlags.CancelSpan) == SpanExitFlags.CancelSpan && span.Exit != null) {
+						bool foundEnd = false;
+						for (int k = i + match.Length; k < CurText.Length; k++) {
+							if (span.Exit.TryMatch (CurText, k - StartOffset).Success)
+								return;
+							if (span.End.TryMatch (CurText, k - StartOffset).Success) {
+								foundEnd = true;
+								break;
 							}
 						}
-						if (mismatch)
-							continue;
-						OnFoundSpanBegin (span, i, match.Length);
-						i += match.Length - 1;
-						spanStack.Push (span);
-						ruleStack.Push (GetRule (span));
-						return;
+						if (!foundEnd)
+							return;
 					}
+					
+					bool mismatch = false;
+					if ((span.BeginFlags & SpanBeginFlags.FirstNonWs) == SpanBeginFlags.FirstNonWs)
+						mismatch = CurText.Take (i).Any (ch => !char.IsWhiteSpace (ch));
+					if (mismatch)
+						continue;
+					FoundSpanBegin (span, i, match.Length);
+					i += match.Length - 1;
+					return;
 				}
 			}
-			
+
 			protected virtual bool ScanSpanEnd (Span cur, ref int i)
 			{
+				int textOffset = i - StartOffset;
+				
 				if (cur.End != null) {
-					RegexMatch match = cur.End.TryMatch (doc, i);
+					RegexMatch match = cur.End.TryMatch (CurText, textOffset);
 					if (match.Success) {
-						OnFoundSpanEnd (cur, i, match.Length);
-						spanStack.Pop ();
-						if (ruleStack.Count > 1) // rulStack[1] is always syntax mode
-							ruleStack.Pop ();
+						FoundSpanEnd (cur, i, match.Length);
+						i += match.Length - 1;
 						return true;
 					}
 				}
-				
+
 				if (cur.Exit != null) {
-					RegexMatch match = cur.Exit.TryMatch (doc, i);
+					RegexMatch match = cur.Exit.TryMatch (CurText, textOffset);
 					if (match.Success) {
-						spanStack.Pop ();
-						if (ruleStack.Count > 1) // rulStack[1] is always syntax mode
-							ruleStack.Pop ();
-						OnFoundSpanExit (cur, i, match.Length);
+						FoundSpanExit (cur, i, match.Length);
+						i += match.Length - 1;
 						return true;
 					}
 				}
 				return false;
 			}
 			
+			public string CurText {
+				get;
+				private set;
+			}
+			public int StartOffset {
+				get;
+				private set;
+			}
+
 			public void ParseSpans (int offset, int length)
 			{
-				maxEnd = System.Math.Min (doc.Length, System.Math.Min (offset + length, line != null ? System.Math.Min (line.Offset + line.EditableLength, doc.Length) : doc.Length));
+				if (offset < 0 || offset + length > doc.Length || length <= 0)
+					return;
+				StartOffset = offset;
+				CurText = doc.GetTextAt (offset, length);
+				maxEnd = offset + length;
 				for (int i = offset; i < maxEnd; i++) {
+					int textIndex = i - StartOffset;
 					Span cur = CurSpan;
 					if (cur != null) {
 						if (cur.Escape != null) {
 							bool mismatch = false;
 							for (int j = 0; j < cur.Escape.Length; j++) {
-								if (i + j >= doc.Length || doc.GetCharAt (i + j) != cur.Escape[j]) {
+								if (textIndex + j >= CurText.Length || CurText[textIndex + j] != cur.Escape[j]) {
 									mismatch = true;
 									break;
 								}
@@ -409,71 +448,57 @@ namespace Mono.TextEditor.Highlighting
 						if (ScanSpanEnd (cur, ref i))
 							continue;
 					}
-					
 					ScanSpan (ref i);
-					
 					if (i < doc.Length)
-						OnParseChar (ref i, doc.GetCharAt (i));
+						ParseChar (ref i, CurText[textIndex]);
 				}
 			}
 		}
-		
+
 		public class ChunkParser
 		{
 			readonly string defaultStyle = "text";
-			SpanParser spanParser;
-			Document doc;
-			LineSegment line;
-			SyntaxMode mode;
-			
+			protected SpanParser spanParser;
+			protected Document doc;
+			protected LineSegment line;
+			internal int lineOffset;
+			protected SyntaxMode mode;
+
 			public ChunkParser (SpanParser spanParser, Document doc, Style style, SyntaxMode mode, LineSegment line)
 			{
 				this.mode = mode;
 				this.doc = doc;
 				this.line = line;
+				this.lineOffset = line.Offset;
 				this.spanParser = spanParser;
-				spanParser.FoundSpanBegin += FoundSpanBegin;
-				spanParser.FoundSpanEnd += FoundSpanEnd;
-				spanParser.FoundSpanExit += FoundSpanExit;
+				spanParser.FoundSpanBegin = FoundSpanBegin;
+				spanParser.FoundSpanEnd = FoundSpanEnd;
+				spanParser.FoundSpanExit = FoundSpanExit;
 				spanParser.ParseChar += ParseChar;
 				if (line == null)
 					throw new ArgumentNullException ("line");
 			}
-			
+
 			Chunk startChunk = null;
 			Chunk endChunk;
-			
+
 			void AddRealChunk (Chunk chunk)
 			{
 				if (startChunk == null) {
 					startChunk = endChunk = chunk;
+					chunk.SpanStack = spanParser.SpanStack.Clone ();
 					return;
 				}
-				if (endChunk.Style.Equals (chunk.Style)) {
+
+				if (endChunk.Style.Equals (chunk.Style) && chunk.SpanStack.Equals (endChunk.SpanStack)) {
 					endChunk.Length += chunk.Length;
 					return;
 				}
+
 				endChunk = endChunk.Next = chunk;
-				/*
-				const int MaxChunkLength = 80;
-				int divisor = chunk.Length / MaxChunkLength;
-				int reminder = chunk.Length % MaxChunkLength;
-				for (int i = 0; i < divisor; i++) {
-					Chunk newChunk = new Chunk (chunk.Offset + i * MaxChunkLength, MaxChunkLength, chunk.Style);
-					if (startChunk == null)
-						startChunk = endChunk = newChunk;
-					else 
-						endChunk = endChunk.Next = newChunk;
-				}
-				if (reminder > 0) {
-					Chunk newChunk = new Chunk (chunk.Offset + divisor * MaxChunkLength, reminder, chunk.Style);
-					if (startChunk == null)
-						startChunk = endChunk = newChunk;
-					else 
-						endChunk = endChunk.Next = newChunk;
-				}*/
+				chunk.SpanStack = spanParser.SpanStack.Clone ();
 			}
-			
+
 			void AddChunk (ref Chunk curChunk, int length, string style)
 			{
 				if (curChunk.Length > 0) {
@@ -488,10 +513,10 @@ namespace Mono.TextEditor.Highlighting
 				curChunk = new Chunk (curChunk.EndOffset, 0, defaultStyle);
 				curChunk.Style = GetSpanStyle ();
 			}
-			
+
 			string GetChunkStyleColor (string topColor)
 			{
-				if (!String.IsNullOrEmpty (topColor)) 
+				if (!String.IsNullOrEmpty (topColor))
 					return topColor;
 				if (String.IsNullOrEmpty (spanParser.SpanStack.Peek ().Color)) {
 					Span span = spanParser.SpanStack.Pop ();
@@ -501,7 +526,7 @@ namespace Mono.TextEditor.Highlighting
 				}
 				return spanParser.SpanStack.Count > 0 ? spanParser.SpanStack.Peek ().Color : defaultStyle;
 			}
-			
+
 			string GetSpanStyle ()
 			{
 				if (spanParser.SpanStack.Count == 0)
@@ -517,72 +542,83 @@ namespace Mono.TextEditor.Highlighting
 					return defaultStyle;
 				return spanParser.SpanStack.Peek ().Color;
 			}
-			
+
 			Chunk curChunk;
-			
-			string GetChunkStyle (Span span) 
+
+			string GetChunkStyle (Span span)
 			{
 				if (span == null)
 					return GetSpanStyle ();
-				return span.TagColor ?? span.Color ?? GetSpanStyle ();
+				return span.Color ?? GetSpanStyle ();
 			}
-			
+
 			public void FoundSpanBegin (Span span, int offset, int length)
 			{
 				curChunk.Length = offset - curChunk.Offset;
-				curChunk.Style  = GetStyle (curChunk) ?? GetSpanStyle ();
-				AddChunk (ref curChunk, 0, curChunk.Style);
-				
-				curChunk.Offset = offset;
-				curChunk.Length = length;
-				curChunk.Style  = GetChunkStyle (span);
+				curChunk.Style  = GetStyle (curChunk);
+				if (string.IsNullOrEmpty (curChunk.Style)) {
+					Span tmpSpan = spanParser.SpanStack.Count > 0 ? spanParser.SpanStack.Pop () : null;
+					curChunk.Style = GetSpanStyle ();
+					if (tmpSpan != null)
+						spanParser.SpanStack.Push (tmpSpan);
+				}
 				AddChunk (ref curChunk, 0, curChunk.Style);
+
 				Rule spanRule = spanParser.GetRule (span);
+				
 				if (spanRule == null)
 					throw new Exception ("Rule " + span.Rule + " not found in " + span);
+				spanParser.PushSpan (span, spanRule);
+
+				curChunk.Offset = offset;
+				curChunk.Length = length;
+				curChunk.Style  = span.TagColor ?? GetChunkStyle (span);
+				curChunk.SpanStack.Push (span);
+				AddChunk (ref curChunk, 0, curChunk.Style);
 				foreach (SemanticRule semanticRule in spanRule.SemanticRules) {
 					semanticRule.Analyze (this.doc, line, curChunk, offset, line.EndOffset);
 				}
 			}
-			
+
 			public void FoundSpanExit (Span span, int offset, int length)
 			{
 				curChunk.Length = offset - curChunk.Offset;
 				AddChunk (ref curChunk, 0, GetChunkStyle (span));
+				spanParser.PopSpan ();
+
 			}
-			
+
 			public void FoundSpanEnd (Span span, int offset, int length)
 			{
 				curChunk.Length = offset - curChunk.Offset;
 				curChunk.Style  = GetStyle (curChunk) ?? GetChunkStyle (span);
 				AddChunk (ref curChunk, 0, defaultStyle);
-				
+
 				curChunk.Offset = offset;
 				curChunk.Length = length;
-				curChunk.Style  = GetChunkStyle (span);
+				curChunk.Style  = span.TagColor ?? GetChunkStyle (span);
 				AddChunk (ref curChunk, 0, defaultStyle);
+				spanParser.PopSpan ();
 			}
-			
+
 			bool inWord = false;
 			public void ParseChar (ref int i, char ch)
 			{
-				int textOffset = i - line.Offset;
-				if (textOffset >= str.Length)
-					return;
-				
+				int textOffset = i - spanParser.StartOffset;
+
 				Rule cur = spanParser.CurRule;
 				bool isWordPart = cur.Delimiter.IndexOf (ch) < 0;
-				
-				if (inWord && !isWordPart || !inWord && isWordPart) 
+
+				if (inWord && !isWordPart || !inWord && isWordPart)
 					AddChunk (ref curChunk, 0, curChunk.Style = GetStyle (curChunk) ?? GetSpanStyle ());
-				
+
 				inWord = isWordPart;
-				
+
 				if (cur.HasMatches && i - curChunk.Offset == 0) {
 					Match foundMatch = null;
 					int   foundMatchLength = 0;
 					foreach (Match ruleMatch in cur.Matches) {
-						int matchLength = ruleMatch.TryMatch (str, textOffset);
+						int matchLength = ruleMatch.TryMatch (spanParser.CurText, textOffset);
 						if (foundMatchLength < matchLength) {
 							foundMatch = ruleMatch;
 							foundMatchLength = matchLength;
@@ -591,40 +627,29 @@ namespace Mono.TextEditor.Highlighting
 					if (foundMatch != null) {
 						AddChunk (ref curChunk, foundMatchLength, GetChunkStyleColor (foundMatch.Color));
 						i += foundMatchLength - 1;
-						curChunk.Length = i - curChunk.Offset + 1; 
+						curChunk.Length = i - curChunk.Offset + 1;
 						return;
 					}
 				}
-				
-				curChunk.Length = i - curChunk.Offset + 1; 
+
+				curChunk.Length = i - curChunk.Offset + 1;
 			}
-			
-			string GetStyle (Chunk chunk)
+
+			protected virtual string GetStyle (Chunk chunk)
 			{
 				if (chunk.Length > 0) {
-/*					Console.WriteLine (spanParser.CurSpan  + " / " + spanParser.CurRule);
-					Console.WriteLine ("----:Span:");
-					foreach (var o in spanParser.SpanStack) {
-						Console.WriteLine (o);
-					}
-					Console.WriteLine ("----:Rule:");
-					foreach (var o in spanParser.RuleStack) {
-						Console.WriteLine (o);
-					}
-					Console.WriteLine ("----");*/
-					Keywords keyword = spanParser.CurRule.GetKeyword (doc, chunk.Offset, chunk.Length); 
+					Keywords keyword = spanParser.CurRule.GetKeyword (doc, chunk.Offset, chunk.Length);
 					if (keyword != null)
 						return keyword.Color;
 				}
 				return null;
 			}
-			
-			string str;
+
 			public virtual Chunk GetChunks (int offset, int length)
 			{
-				SyntaxModeService.ScanSpans (doc, mode, spanParser.CurRule, spanParser.SpanStack, line.Offset, offset);
+				if (lineOffset < offset)
+					SyntaxModeService.ScanSpans (doc, mode, spanParser.CurRule, spanParser.SpanStack, lineOffset, offset);
 				length = System.Math.Min (doc.Length - offset, length);
-				str = length > 0 ? doc.GetTextAt (offset, length) : null;
 				curChunk = new Chunk (offset, 0, GetSpanStyle ());
 				spanParser.ParseSpans (offset, length);
 				curChunk.Length = offset + length - curChunk.Offset;
@@ -635,7 +660,7 @@ namespace Mono.TextEditor.Highlighting
 				return startChunk;
 			}
 		}
-		
+
 		public override Rule GetRule (string name)
 		{
 			if (name == null || name == "<root>") {
@@ -643,30 +668,30 @@ namespace Mono.TextEditor.Highlighting
 			}
 			if (name.StartsWith ("mode:"))
 				return SyntaxModeService.GetSyntaxMode (name.Substring ("mode:".Length));
-			
+
 			foreach (Rule rule in rules) {
 				if (rule.Name == name)
 					return rule;
 			}
 			return this;
 		}
-		
+
 		void AddSemanticRule (Rule rule, SemanticRule semanticRule)
 		{
 			if (rule != null)
 				rule.SemanticRules.Add (semanticRule);
 		}
-		
+
 		public void AddSemanticRule (SemanticRule semanticRule)
 		{
 			AddSemanticRule (this, semanticRule);
 		}
-		
+
 		public void AddSemanticRule (string addToRuleName, SemanticRule semanticRule)
 		{
 			AddSemanticRule (GetRule (addToRuleName), semanticRule);
 		}
-		
+
 		void RemoveSemanticRule (Rule rule, Type type)
 		{
 			if (rule != null) {
@@ -686,16 +711,16 @@ namespace Mono.TextEditor.Highlighting
 		{
 			RemoveSemanticRule (GetRule (removeFromRuleName), type);
 		}
-		
+
 		public override string ToString ()
 		{
 			return String.Format ("[SyntaxMode: Name={0}, MimeType={1}]", Name, MimeType);
 		}
-				
-		new const string Node = "SyntaxMode"; 
-		
+
+		new const string Node = "SyntaxMode";
+
 		public const string MimeTypesAttribute = "mimeTypes";
-		
+
 		public static SyntaxMode Read (XmlReader reader)
 		{
 			SyntaxMode result = new SyntaxMode ();
