@@ -33,6 +33,7 @@ namespace MonoDevelop.Core.Execution
 {
 	public interface IExecutionHandler
 	{
-		IProcessAsyncOperation Execute (string command, string arguments, string workingDirectory, IDictionary<string, string> environmentVariables, IConsole console);
+		bool CanExecute (ExecutionCommand command);
+		IProcessAsyncOperation Execute (ExecutionCommand command, IConsole console);
 	}
 }
