@@ -26,14 +26,15 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 			}
 		}
 		
-		public SplashScreenForm () : base (Gtk.WindowType.Toplevel)
+		//this is a popup so it behaves like other splashes on Windows, i.e. doesn't show up as a second window in the taskbar.
+		public SplashScreenForm () : base (Gtk.WindowType.Popup)
 		{
 			AppPaintable = true;
 			this.Decorated = false;
 			this.WindowPosition = WindowPosition.Center;
 			this.TypeHint = Gdk.WindowTypeHint.Splashscreen;
 			try {
-				bitmap = new Gdk.Pixbuf(Assembly.GetCallingAssembly(), "SplashScreen.png");
+				bitmap = new Gdk.Pixbuf (Assembly.GetCallingAssembly(), "SplashScreen.png");
 			} catch (Exception e) {
 				LoggingService.LogError ("Can't load splash screen pixbuf 'SplashScreen.png'.", e);
 			}
@@ -53,19 +54,30 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 				this.Resize (bitmap.Width, bitmap.Height);
 		}
 		
+		protected override void OnDestroyed ()
+		{
+			base.OnDestroyed ();
+			if (bitmap != null) {
+				bitmap.Dispose ();
+				bitmap = null;
+			}
+		}
+		
 		protected override bool OnExposeEvent (Gdk.EventExpose evt)
 		{
 			if (bitmap != null) {
 				Gdk.GC gc = Style.LightGC (StateType.Normal);
 				GdkWindow.DrawPixbuf (gc, bitmap, 0, 0, 0, 0, bitmap.Width, bitmap.Height, Gdk.RgbDither.None, 0, 0);
 
-				Pango.Layout pl = new Pango.Layout (PangoContext);
-				Pango.FontDescription des = this.Style.FontDescription.Copy();
-				pl.FontDescription = des;
-				//pl.SetMarkup("<b><span foreground='#cccccc'>" + BuildVariables.PackageVersionLabel + "</span></b>");
-				int w,h;
-				pl.GetPixelSize (out w, out h);
-				GdkWindow.DrawLayout (gc, bitmap.Width - w - 75, 90, pl);
+				using (Pango.Layout pl = new Pango.Layout (PangoContext)) {
+					Pango.FontDescription des = this.Style.FontDescription.Copy();
+					pl.FontDescription = des;
+					//pl.SetMarkup("<b><span foreground='#cccccc'>" + BuildVariables.PackageVersionLabel + "</span></b>");
+					int w,h;
+					pl.GetPixelSize (out w, out h);
+					GdkWindow.DrawLayout (gc, bitmap.Width - w - 75, 90, pl);
+					des.Dispose ();
+				}
 			}
 			return base.OnExposeEvent (evt);
 		}
@@ -78,17 +90,17 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 
 		public void SetMessage (string Message)
 		{
-			if (bitmap == null) {
+/*			if (bitmap == null) {
 				label.Text = Message;
 			} else {
 				label.Markup = "<span size='small' foreground='white'>" + Message + "</span>";
 			}
-			RunMainLoop ();
+			RunMainLoop ();*/
 		}
 		
 		static void RunMainLoop ()
 		{
-			MonoDevelop.Core.Gui.DispatchService.RunPendingEvents ();
+			DispatchService.RunPendingEvents ();
 		}
 		
 		void IProgressMonitor.BeginTask (string name, int totalWork)
@@ -153,6 +165,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 		void IDisposable.Dispose ()
 		{
 			Destroy ();
+			splashScreen = null;
 		}
 	}
 }
