@@ -64,7 +64,7 @@ namespace MonoDevelop.Ide.Gui
 				statusBar.ShowMessage (icon, CurrentTask);
 			if (!UnknownWork)
 				statusBar.SetProgressFraction (GlobalWork);
-			DispatchService.RunPendingEvents ();
+			RunPendingEvents ();
 		}
 		
 		public void UpdateStatusBar ()
@@ -97,16 +97,9 @@ namespace MonoDevelop.Ide.Gui
 				
 				base.OnCompleted ();
 				
-				if (showErrorDialogs) {
-					MultiMessageDialog resultDialog = new MultiMessageDialog ();
-					foreach (string m in Errors)
-						resultDialog.AddError (m);
-					foreach (string m in Warnings)
-						resultDialog.AddWarning (m);
-					resultDialog.TransientFor = IdeApp.Workbench.RootWindow;
-					resultDialog.Run ();
-					resultDialog.Destroy ();
-				}
+				if (showErrorDialogs)
+					ShowResultDialog ();
+				
 				IdeApp.Workbench.StatusBar.SetMessageSourcePad (statusSourcePad);
 				return;
 			}
