@@ -54,9 +54,9 @@ namespace MonoDevelop.Ide.Templates
 		bool showAutogenerationNotice = false;
 		string sourceLang;
 		
-		public override void Load (XmlElement filenode)
+		public override void Load (XmlElement filenode, FilePath baseDirectory)
 		{
-			base.Load (filenode);
+			base.Load (filenode, baseDirectory);
 			content = filenode.InnerText;
 			
 			sourceLang = filenode.GetAttribute ("SourceLanguage");
@@ -151,8 +151,7 @@ namespace MonoDevelop.Ide.Templates
 			//This is a bit hacky doing it here instead of in CreateContent, but need to
 			//substitute all tags in code before language is translated, because language
 			//translation gets confused by unsubstituted  substitution tokens.
-			string [,] tagsArr = HashtableToStringArray (tags);
-			tempSubstitutedContent = StringParserService.Parse (content, tagsArr);
+			tempSubstitutedContent = StringParserService.Parse (content, tags);
 		}
 		
 		private System.CodeDom.Compiler.CodeDomProvider GetCodeDomProvider (string language)
