@@ -50,7 +50,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 	public class FindMemberVisitorTests : UnitTests.TestBase
 	{
 		#region TestHelper
-		static NRefactoryParser parser = new NRefactoryParser ();
+		static McsParser parser = new McsParser ();
 		
 		void RunTest (string test)
 		{
@@ -97,7 +97,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 		//	RefactorerContext ctx = new RefactorerContext (dom, new DumbTextFileProvider(testViewContent), null);
 			NRefactoryResolver resolver = new NRefactoryResolver (dom, 
 			                                                      parsedDocument.CompilationUnit, 
-			                                                      MonoDevelop.Ide.Gui.TextEditor.GetTextEditor (testViewContent), 
+			                                                      testViewContent.Data, 
 			                                                      "a.cs");
 			SearchMemberVisitor smv = new SearchMemberVisitor (memberLocation.Line);
 			if (localVariable != null) {
@@ -118,8 +118,8 @@ namespace MonoDevelop.CSharpBinding.Tests
 				                               ((IType)smv.FoundMember).TypeParameters.Count,
 				                               true);
 			}
-			FindMemberAstVisitor astVisitor = new FindMemberAstVisitor (resolver, testViewContent, smv.FoundMember);
-			astVisitor.RunVisitor ();
+			FindMemberAstVisitor astVisitor = new FindMemberAstVisitor (testViewContent.GetTextEditorData ().Document, smv.FoundMember);
+			astVisitor.RunVisitor (resolver);
 			
 			int i = 0, j = 0;
 			StringBuilder errorText = new StringBuilder ();
@@ -131,7 +131,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 					if (expectedReferences[i].Line < astVisitor.FoundReferences[j].Line) {
 						errorText.Append ("Reference at  line " + expectedReferences[i].Line + " not found.");
 						errorText.AppendLine ();
-						errorText.Append (doc.GetTextAt (doc.GetLine (expectedReferences[i].Line - 1)).Replace ('\t', ' '));
+						errorText.Append (doc.GetTextAt (doc.GetLine (expectedReferences[i].Line)).Replace ('\t', ' '));
 						errorText.AppendLine ();
 						errorText.Append (new string (' ', expectedReferences[i].Column));errorText.Append ('^');
 						errorText.AppendLine ();
@@ -141,7 +141,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 					if (expectedReferences[i].Line > astVisitor.FoundReferences[j].Line) {
 						errorText.Append ("Found unexpected Reference at line " + astVisitor.FoundReferences[j].Line);
 						errorText.AppendLine ();
-						errorText.Append (doc.GetTextAt (doc.GetLine (astVisitor.FoundReferences[j].Line - 1)).Replace ('\t', ' '));
+						errorText.Append (doc.GetTextAt (doc.GetLine (astVisitor.FoundReferences[j].Line)).Replace ('\t', ' '));
 						errorText.AppendLine ();
 						errorText.Append (new string (' ', astVisitor.FoundReferences[j].Column));errorText.Append ('^');
 						errorText.AppendLine ();
@@ -151,7 +151,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 					
 					errorText.Append ("Column mismatch at line " + astVisitor.FoundReferences[j].Line + " was: " + astVisitor.FoundReferences[j].Column + " should be:" + expectedReferences[i].Column);
 					errorText.AppendLine ();
-					errorText.Append (doc.GetTextAt (doc.GetLine (astVisitor.FoundReferences[j].Line - 1)).Replace ('\t', ' '));
+					errorText.Append (doc.GetTextAt (doc.GetLine (astVisitor.FoundReferences[j].Line)).Replace ('\t', ' '));
 					errorText.Append (new string (' ', expectedReferences[i].Column));errorText.Append ('^');
 					errorText.AppendLine ();
 					errorText.Append (new string (' ', astVisitor.FoundReferences[j].Column));errorText.Append ('^');
@@ -162,7 +162,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 			while (i < expectedReferences.Count) {
 				errorText.Append ("Reference at  line " + expectedReferences[i].Line + " not found.");
 				errorText.AppendLine ();
-				errorText.Append (doc.GetTextAt (doc.GetLine (expectedReferences[i].Line - 1)).Replace ('\t', ' '));
+				errorText.Append (doc.GetTextAt (doc.GetLine (expectedReferences[i].Line)).Replace ('\t', ' '));
 				errorText.AppendLine ();
 				errorText.Append (new string (' ', expectedReferences[j].Column));errorText.Append ('^');
 				errorText.AppendLine ();
@@ -171,7 +171,7 @@ namespace MonoDevelop.CSharpBinding.Tests
 			while (j < astVisitor.FoundReferences.Count) {
 				errorText.Append ("Found unexpected Reference at line " + astVisitor.FoundReferences[j].Line);
 				errorText.AppendLine ();
-				errorText.Append (doc.GetTextAt (doc.GetLine (astVisitor.FoundReferences[j].Line - 1)).Replace ('\t', ' '));
+				errorText.Append (doc.GetTextAt (doc.GetLine (astVisitor.FoundReferences[j].Line)).Replace ('\t', ' '));
 				errorText.AppendLine ();
 				errorText.Append (new string (' ', astVisitor.FoundReferences[i].Column));errorText.Append ('^');
 				errorText.AppendLine ();
@@ -756,6 +756,73 @@ object item)
 ");
 		}
 
+		/// <summary>
+		/// Bug 615702 - In-place variable renaming can't rename foreach loop variables
+		/// </summary>
+		[Test()]
+		public void TestBug615702 ()
+		{
+			LocalVariable localVariable = new LocalVariable (null,
+			                                  "obj",
+			                                  DomReturnType.Int32,
+			                                  new DomRegion (5, 3, 8, 3));
+			RunTest (
+@"class FooBar
+{
+	public static void Main (string[] args)
+	{
+		foreach (object @obj in new object[3])
+		{
+			Console.WriteLine (@obj.GetType());
+		}
+	}
+}", localVariable);
+		}
+		
+		
+		/// <summary>
+		/// Bug 615983 - Refactoring does not include object initializers
+		/// </summary>
+		[Test()]
+		public void TestBug615983 ()
+		{
+			RunTest (
+@"class test
+{
+	public string $@property { get; set; }
+
+	void Test ()
+	{
+		test product = new test {
+			@property = ""asdf""
+		};
+		product.@property = ""asdf"";
+	}
+}");
+		}
+		
+		/// <summary>
+		/// Bug 693228 - Rename in body of foreach loop doesn't change declaration instance
+		/// </summary>
+		[Test()]
+		public void TestBug693228 ()
+		{
+			LocalVariable localVariable = new LocalVariable (null,
+			                                  "arg",
+			                                  DomReturnType.String,
+			                                  new DomRegion (4, 29, 6, 9));
+			RunTest (
+@"class TestClass {
+	public static void Main (string[] args)
+	{
+		foreach (var $@arg in args) {
+			Console.WriteLine (@arg);
+		}
+		
+	}
+}
+", localVariable);
+		}
 		
 		/*
 		[Test()]
